#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

"""
The Model Zoo.

This file contains a list of all the models in the model zoo, the path to
load them, agents & tasks associated (e.g. they were trained using) and a
description. Using the path you should be able to download and use the model
automatically, e.g.:

... code-block:

   parlai interactive --model-file
       "zoo:wikipedia_20161221/tfidf_retriever/drqa_docs"


There are a number of guidelines you should follow in the zoo:

- You should choose the best directory name as possible. An input of
  ``zoo:PROJECTNAME/MODELNAME/FILENAME`` will attempt to use a build script from
  parlai/zoo/PROJECTNAME/MODELNAME.py.
- You should include all of the following fields:
    * title: the name of the entry:
    * id: corresponds to PROJECTNAME
    * description: describe the entry in reasonable detail. It should be at least
      a couple sentences.
    * example: an example command to chat with or evaluate the model
    * result: the expected output from running the model. You are strongly encouraged
      to make a nightly test which verifies this result.
    * external_website: if applicable, an external website related to the zoo to
      link to.
    * project: if applicable, a link to the project folder. You must have either
      external_website or project.
    * example2 and result2 (optional): additional examples to run.

- As much as possible, you should try to include two examples: one to generate
  some key metrics (e.g. from a paper) and one to actually chat with the model
  using interactive.py. Both should strongly attempt to minimize mandatory
  command line flags.
"""

model_list = [
    {
        "title": "DrQA SQuAD model",
        "id": "drqa",
        "path": "zoo:drqa/squad/model",
        "agent": "drqa",
        "task": "squad",
        "description": "DrQA Reader trained on SQuAD",
        "external_website": "https://github.com/facebookresearch/DrQA",
        "example": "parlai eval_model -mf zoo:drqa/squad/model -t squad -dt test",
        "result": (
            # TODO: this differs slightly from the actual results as of 2019-07-23
            "{'exs': 10570, 'accuracy': 0.6886, 'f1': 0.7821, 'hits@1': 0.689, 'hits@5': 0.689, 'hits@10': 0.689, 'hits@100': 0.689, 'bleu': 0.1364, 'train_loss': 0}"  # noqa: E501
        ),
    },
    {
        "title": "Wikipedia Retriever (used for open SQuAD)",
        "id": "wikipedia_20161221",
        "path": "zoo:wikipedia_20161221/tfidf_retriever/drqa_docs",
        "agent": "tfidf_retriever",
        "external_website": "https://github.com/facebookresearch/DrQA",
        "task": "wikipedia:full",
        "example": (
            "parlai interactive --model tfidf_retriever "
            "-mf zoo:wikipedia_20161221/tfidf_retriever/drqa_docs"
        ),
        "result": (
            """
            Enter Your Message: Yann LeCun
            [candidate_scores]: [507.05804682 390.18244433 279.24033928 269.60377042 214.00140589]
            [SparseTfidfRetrieverAgent]:
            Deep learning (also known as deep structured learning, hierarchical learning or deep machine learning) is a branch of machine learning based on a set of algorithms that attempt to model high level abstractions in data. In a simple case, you could have two sets of neurons: ones that receive an input signal and ones that send an output signal. When the input layer receives an input it passes on a modified version of the input to the next layer. In a deep network, there are many layers between the input and output (and the layers are not made of neurons but it can help to think of it that way), allowing the algorithm to use multiple processing layers, composed of multiple linear and non-linear transformations.

            Deep learning is part of a broader family of machine learning methods based on ...
            to commonsense reasoning which operates on concepts in terms of production rules of the grammar, and is a basic goal of both human language acquisition and AI. (See also Grammar induction.)
            """  # noqa: E501
        ),
        "description": (
            "Retrieval over Wikipedia dump, used for DrQA on the open squad "
            "dataset. This is the dump from the original paper, used for "
            "replicating results."
        ),
    },
    {
        "title": "Wikipedia Retriever (used for Wizard of Wikipedia)",
        "id": "wikipedia_full",
        "path": "zoo:wikipedia_full/tfidf_retriever/model",
        "agent": "tfidf_retriever",
        "task": "wikipedia:full",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/wizard_of_wikipedia",
        "description": (
            "Retrieval over Wikipedia dump, used for DrQA on the open squad dataset."
        ),
        "example": (
            "parlai interactive --model tfidf_retriever -mf "
            "zoo:wikipedia_full/tfidf_retriever/model"
        ),
        "result": (
            """
            Enter Your Message: Yann LeCun
            [candidate_scores]: [454.74038503 353.88863708 307.31353203 280.4501096  269.89960432]
            [SparseTfidfRetrieverAgent]:
            Yann LeCun (; born 1960) is a computer scientist with contributions in machine learning, computer vision, mobile robotics and computational neuroscience. He is well known for his work on optical character recognition and computer vision using convolutional neural networks (CNN), and is a founding father of convolutional nets. He is also one of the main creators of the DjVu image compression technology (together with Léon Bottou and Patrick Haffner). He co-developed the Lush programming language with Léon Bottou.

            Yann LeCun was born near Paris, France, in 1960. He received a Diplôme d'Ingénieur from the Ecole Superieure d'Ingénieur en Electrotechnique et Electronique (ESIEE), Paris in 1983, and a PhD in Computer Science from Université Pierre et Marie Curie in 1987 during which he ...
            of Science and Technology in Saudi Arabia because he was considered a terrorist in the country in view of his atheism.

            In 2018 Yann LeCun picked a fight with a robot to support Facebook AI goals.
            """  # noqa: E501
        ),
    },
    {
        "title": "Wizard of Wikipedia (End to end Generator)",
        "id": "wizard_of_wikipedia",
        "path": "zoo:wizard_of_wikipedia/end2end_generator/model",
        "description": "End2End Generative model for Wizard of Wikipedia",
        "task": "wizard_of_wikipedia:generator",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/wizard_of_wikipedia",
        "example": (
            "parlai display_model -t wizard_of_wikipedia:generator "
            "-mf zoo:wizard_of_wikipedia/end2end_generator/model -n 1 "
            "--display-ignore-fields knowledge_parsed"
        ),
        "result": (
            """
            [chosen_topic]: Gardening
            [knowledge]: no_passages_used __knowledge__ no_passages_used
            Gardening __knowledge__ Gardening is the practice of growing and cultivating plants as part of horticulture.
            Gardening __knowledge__ In gardens, ornamental plants are often grown for their flowers, foliage, or overall appearance; useful plants, such as root vegetables, leaf vegetables, fruits, and herbs, are grown for consumption, for use as dyes, or for medicinal or cosmetic use.
            Gardening __knowledge__ Gardening is considered by many people to be a relaxing activity.
            Gardening __knowledge__ Gardening ranges in scale from fruit orchards, to long boulevard plantings with one or more different types of shrubs, trees, and herbaceous plants, to residential yards including lawns and foundation plantings, to plants in large or small containers ...
            there had been several other notable gardening magazines in circulation, including the "Gardeners' Chronicle" and "Gardens Illustrated", but these were tailored more for the professional gardener.

            [title]: Gardening
            [checked_sentence]: Gardening is considered by many people to be a relaxing activity.
            [eval_labels_choice]: I live on a farm, we garden all year long, it is very relaxing.
            [checked_sentence_parsed]: Gardening __knowledge__ Gardening is considered by many people to be a relaxing activity.
            [WizTeacher]: Gardening
            I like Gardening, even when I've only been doing it for a short time.
            [eval_labels: I live on a farm, we garden all year long, it is very relaxing.]
            [TorchAgent]: i love gardening , it is considered a relaxing activity .
            """  # noqa: E501
        ),
    },
    {
        "title": "Wizard of Wikipedia (Full Dialogue Retrieval Model)",
        "id": "wizard_of_wikipedia",
        "path": "zoo:wizard_of_wikipedia/full_dialogue_retrieval_model/model",
        "description": "Full Dialogue Retrieval Model for Wizard of Wikipedia",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/wizard_of_wikipedia",
        "task": "wizard_of_wikipedia",
        "example2": "parlai interactive -m projects:wizard_of_wikipedia:interactive_retrieval -t wizard_of_wikipedia",
        "result2": (
            "[ Your chosen topic is: Teapot ]\n"
            "Enter Your Message: do you like tea?\n"
            "[WizardRetrievalInteractiveAgent]: Yes!  I only use teapots that have a little air hole in the lid. That prevents the spout from dripping or splashing when the tea is poured. Most teapots have this though.\n"
            "Enter Your Message: what about kettles?\n"
            "[WizardRetrievalInteractiveAgent]: I would think you could use them to heat any type of liquid! I use my teapots with a tea cosy. It's a thermal cover that helps keep the tea hot.\n"
            "Enter Your Message: do you like earl grey?\n"
            "[WizardRetrievalInteractiveAgent]: I think I'll try some Lipton, I love their green tea!"
        ),
        "example": (
            "parlai display_model -t wizard_of_wikipedia "
            "-mf zoo:wizard_of_wikipedia/full_dialogue_retrieval_model/model "
            "-m projects:wizard_of_wikipedia:wizard_transformer_ranker "
            "--n-heads 6 --ffn-size 1200 --embeddings-scale False "
            "--delimiter ' __SOC__ ' --n-positions 1000 --legacy True "
        ),
        "result": (
            """
            [chosen_topic]: Gardening
            [knowledge]: Gardening Gardening is the practice of growing and cultivating plants as part of horticulture.
            Gardening In gardens, ornamental plants are often grown for their flowers, foliage, or overall appearance; useful plants, such as root vegetables, leaf vegetables, fruits, and herbs, are grown for consumption, for use as dyes, or for medicinal or cosmetic use.
            Gardening Gardening is considered by many people to be a relaxing activity.
            Gardening Gardening ranges in scale from fruit orchards, to long boulevard plantings with one or more different types of shrubs, trees, and herbaceous plants, to residential yards including lawns and foundation plantings, to plants in large or small containers grown inside or outside.
            Gardening Gardening may be very specialized, with only one type of plant grown, ...
            there had been several other notable gardening magazines in circulation, including the "Gardeners' Chronicle" and "Gardens Illustrated", but these were tailored more for the professional gardener.

            [title]: Gardening
            [checked_sentence]: Gardening is considered by many people to be a relaxing activity.
            [eval_labels_choice]: I live on a farm, we garden all year long, it is very relaxing.
            [wizard_of_wikipedia]: Gardening
            I like Gardening, even when I've only been doing it for a short time.
            [label_candidates: OK what's the history?|Right, thats cool. I had no idea they still did the DVD thing, What is Netflix's highest rated show? do you know? |I will definitely check his first album out as he sounds interesting.|I don't know a whole lot about it. I was raised Catholic but don't practice anything now.|Well , this was a good conversation. |...and 95 more]
            [eval_labels: I live on a farm, we garden all year long, it is very relaxing.]
               [TorchAgent]: I live on a farm, we garden all year long, it is very relaxing.
            """  # noqa: E501
        ),
    },
    {
        "title": "LIGHT BERT-Biranker Dialogue model",
        "id": "light",
        "path": "zoo:light/biranker_dialogue/model",
        "agent": "bert_ranker/bi_encoder_ranker",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/light",
        "task": "light_dialog",
        "description": "LIGHT Dialogue task, replicating the numbers from the paper.",
        "example": (
            "parlai eval_model -t light_dialog -mf zoo:light/biranker_dialogue/model"
        ),
        "result": "{'exs': 6623, 'accuracy': 0.7586, 'f1': 0.7802, 'hits@1': 0.759, 'hits@5': 0.965,"  # noqa: E501
        "'hits@10': 0.994, 'hits@100': 1.0, 'bleu': 0.7255, 'lr': 5e-05, 'total_train_updates': 15050,"  # noqa: E501
        "'examples': 6623, 'loss': 5307.0, 'mean_loss': 0.8013, 'mean_rank': 1.599, 'train_accuracy': 0}",  # noqa: E501
    },
    {
        "title": "Controllable Dialogue ConvAI2 model",
        "id": "controllable_dialogue",
        "path": "zoo:controllable_dialogue/convai2_finetuned_baseline",
        "agent": "projects.controllable_dialogue.controllable_seq2seq.controllable_seq2seq:ControllableSeq2seqAgent",  # noqa: E501
        "task": "convai2",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/controllable_dialogue",
        "example": (
            "parlai eval_model --model "
            "projects.controllable_dialogue.controllable_seq2seq.controllable_seq2seq:"
            "ControllableSeq2seqAgent --task "
            "projects.controllable_dialogue.tasks.agents "
            "-mf zoo:controllable_dialogue/convai2_finetuned_baseline"
        ),
        "result": (
            "{'exs': 7801, 'accuracy': 0.0006409, 'f1': 0.1702, 'bleu': 0.005205, "
            "'token_acc': 0.3949, 'loss': 3.129, 'ppl': 22.86}"
        ),
        "description": "Seq2Seq model with control trained on ConvAI2",
    },
    {
        "title": "TransResNet (ResNet 152) Personality-Captions model",
        "id": "personality_captions",
        "path": "zoo:personality_captions/transresnet",
        "agent": "projects.personality_captions.transresnet.transresnet:TransresnetAgent",  # noqa: E501
        "task": "personality_captions",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/personality_captions",
        "description": (
            "Transresnet Model pretrained on the Personality-Captions task"
        ),
        "example": (
            "parlai eval_model -t personality_captions "
            "-mf zoo:personality_captions/transresnet/model --num-test-labels 5 -dt test"
        ),
        "result": (
            "{'exs': 10000, 'accuracy': 0.5113, 'f1': 0.5951, 'hits@1': 0.511, "
            "'hits@5': 0.816, 'hits@10': 0.903, 'hits@100': 0.998, 'bleu': 0.4999, "
            "'hits@1/100': 1.0, 'loss': -0.002, 'med_rank': 1.0}"
        ),
    },
    {
        "title": "Poly-Encoder Transformer Reddit Pretrained Model",
        "id": "pretrained_transformers",
        "path": "zoo:pretrained_transformers/poly_model_huge_reddit",
        "agent": "transformer/polyencoder",
        "task": "pretrained_transformers",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/polyencoder/",
        "description": (
            "Poly-Encoder pretrained on Reddit. Use this model as an ``--init-model`` for a poly-encoder "
            "when fine-tuning on another task. For more details on how to train, see the project page."
        ),
        "example": (
            "parlai train_model "
            "--init-model zoo:pretrained_transformers/poly_model_huge_reddit/model "
            "-t convai2 "
            "--model transformer/polyencoder --batchsize 256 --eval-batchsize 10 "
            "--warmup_updates 100 --lr-scheduler-patience 0 --lr-scheduler-decay 0.4 "
            "-lr 5e-05 --data-parallel True --history-size 20 --label-truncate 72 "
            "--text-truncate 360 --num-epochs 8.0 --max_train_time 200000 -veps 0.5 "
            "-vme 8000 --validation-metric accuracy --validation-metric-mode max "
            "--save-after-valid True --log_every_n_secs 20 --candidates batch --fp16 True "
            "--dict-tokenizer bpe --dict-lower True --optimizer adamax --output-scaling 0.06 "
            "--variant xlm --reduction-type mean --share-encoders False "
            "--learn-positional-embeddings True --n-layers 12 --n-heads 12 --ffn-size 3072 "
            "--attention-dropout 0.1 --relu-dropout 0.0 --dropout 0.1 --n-positions 1024 "
            "--embedding-size 768 --activation gelu --embeddings-scale False --n-segments 2 "
            "--learn-embeddings True --polyencoder-type codes --poly-n-codes 64 "
            "--poly-attention-type basic --dict-endtoken __start__ "
            "--model-file <YOUR MODEL FILE>"
        ),
        "result": (
            "(subject to some variance, you may see the following as a result of validation of the model)\n"
            "{'exs': 7801, 'accuracy': 0.8942 ...}"
        ),
    },
    {
        "title": "Poly-Encoder Transformer Wikipedia/Toronto Books Pretrained Model",
        "id": "pretrained_transformers",
        "path": "zoo:pretrained_transformers/poly_model_huge_wikito",
        "agent": "transformer/polyencoder",
        "task": "pretrained_transformers",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/polyencoder/",
        "description": (
            "Poly-Encoder pretrained on Wikipedia/Toronto Books. Use this model as an ``--init-model`` for a poly-encoder "
            "when fine-tuning on another task. For more details on how to train, see the project page."
        ),
        "example": (
            "parlai train_model "
            "--init-model zoo:pretrained_transformers/poly_model_huge_wikito/model "
            "-t convai2 "
            "--model transformer/polyencoder --batchsize 256 --eval-batchsize 10 "
            "--warmup_updates 100 --lr-scheduler-patience 0 --lr-scheduler-decay 0.4 "
            "-lr 5e-05 --data-parallel True --history-size 20 --label-truncate 72 "
            "--text-truncate 360 --num-epochs 8.0 --max_train_time 200000 -veps 0.5 "
            "-vme 8000 --validation-metric accuracy --validation-metric-mode max "
            "--save-after-valid True --log_every_n_secs 20 --candidates batch --fp16 True "
            "--dict-tokenizer bpe --dict-lower True --optimizer adamax --output-scaling 0.06 "
            "--variant xlm --reduction-type mean --share-encoders False "
            "--learn-positional-embeddings True --n-layers 12 --n-heads 12 --ffn-size 3072 "
            "--attention-dropout 0.1 --relu-dropout 0.0 --dropout 0.1 --n-positions 1024 "
            "--embedding-size 768 --activation gelu --embeddings-scale False --n-segments 2 "
            "--learn-embeddings True --polyencoder-type codes --poly-n-codes 64 "
            "--poly-attention-type basic --dict-endtoken __start__ "
            "--model-file <YOUR MODEL FILE>"
        ),
        "result": (
            "(subject to some variance, you may see the following as a result of validation of the model)\n"
            "{'exs': 7801, 'accuracy': 0.861 ...}"
        ),
    },
    {
        "title": "Bi-Encoder Transformer Reddit Pretrained Model",
        "id": "pretrained_transformers",
        "path": "zoo:pretrained_transformers/poly_model_huge_reddit",
        "agent": "transformer/biencoder",
        "task": "pretrained_transformers",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/polyencoder/",
        "description": (
            "Bi-Encoder pretrained on Reddit. Use this model as an ``--init-model`` for a bi-encoder "
            "when fine-tuning on another task. For more details on how to train, see the project page."
        ),
        "example": (
            "parlai train_model "
            "--init-model zoo:pretrained_transformers/bi_model_huge_reddit/model "
            "--batchsize 512 -t convai2 "
            "--model transformer/biencoder --eval-batchsize 6 "
            "--warmup_updates 100 --lr-scheduler-patience 0 "
            "--lr-scheduler-decay 0.4 -lr 5e-05 --data-parallel True "
            "--history-size 20 --label-truncate 72 --text-truncate 360 "
            "--num-epochs 10.0 --max_train_time 200000 -veps 0.5 -vme 8000 "
            "--validation-metric accuracy --validation-metric-mode max "
            "--save-after-valid True --log_every_n_secs 20 --candidates batch "
            "--dict-tokenizer bpe --dict-lower True --optimizer adamax "
            "--output-scaling 0.06 "
            "--variant xlm --reduction-type mean --share-encoders False "
            "--learn-positional-embeddings True --n-layers 12 --n-heads 12 "
            "--ffn-size 3072 --attention-dropout 0.1 --relu-dropout 0.0 --dropout 0.1 "
            "--n-positions 1024 --embedding-size 768 --activation gelu "
            "--embeddings-scale False --n-segments 2 --learn-embeddings True "
            "--share-word-embeddings False --dict-endtoken __start__ --fp16 True "
            "--model-file <YOUR MODEL FILE>"
        ),
        "result": (
            "(subject to some variance, you may see the following as a result of validation of the model)\n"
            "{'exs': 7801, 'accuracy': 0.8686 ...}"
        ),
    },
    {
        "title": "Bi-Encoder Transformer Wikipedia/Toronto Books Pretrained Model",
        "id": "pretrained_transformers",
        "path": "zoo:pretrained_transformers/bi_model_huge_wikito",
        "agent": "transformer/biencoder",
        "task": "pretrained_transformers",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/polyencoder/",
        "description": (
            "Bi-Encoder pretrained on Wikipedia/Toronto Books. Use this model as an ``--init-model`` for a poly-encoder "
            "when fine-tuning on another task. For more details on how to train, see the project page."
        ),
        "example": (
            "parlai train_model "
            "--init-model zoo:pretrained_transformers/bi_model_huge_wikito/model "
            "--batchsize 512 -t convai2 "
            "--model transformer/biencoder --eval-batchsize 6 "
            "--warmup_updates 100 --lr-scheduler-patience 0 "
            "--lr-scheduler-decay 0.4 -lr 5e-05 --data-parallel True "
            "--history-size 20 --label-truncate 72 --text-truncate 360 "
            "--num-epochs 10.0 --max_train_time 200000 -veps 0.5 -vme 8000 "
            "--validation-metric accuracy --validation-metric-mode max "
            "--save-after-valid True --log_every_n_secs 20 --candidates batch "
            "--dict-tokenizer bpe --dict-lower True --optimizer adamax "
            "--output-scaling 0.06 "
            "--variant xlm --reduction-type mean --share-encoders False "
            "--learn-positional-embeddings True --n-layers 12 --n-heads 12 "
            "--ffn-size 3072 --attention-dropout 0.1 --relu-dropout 0.0 --dropout 0.1 "
            "--n-positions 1024 --embedding-size 768 --activation gelu "
            "--embeddings-scale False --n-segments 2 --learn-embeddings True "
            "--share-word-embeddings False --dict-endtoken __start__ --fp16 True "
            "--model-file <YOUR MODEL FILE>"
        ),
        "result": (
            "(subject to some variance, you may see the following as a result of validation of the model)\n"
            "{'exs': 7801, 'accuracy': 0.846 ...}"
        ),
    },
    {
        "title": "Cross-Encoder Transformer Reddit Pretrained Model",
        "id": "pretrained_transformers",
        "path": "zoo:pretrained_transformers/cross_model_huge_reddit",
        "agent": "transformer/crossencoder",
        "task": "pretrained_transformers",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/polyencoder/",
        "description": (
            "Cross-Encoder pretrained on Reddit. Use this model as an ``--init-model`` for a cross-encoder "
            "when fine-tuning on another task. For more details on how to train, see the project page."
        ),
        "example": (
            "parlai train_model "
            "--init-model zoo:pretrained_transformers/cross_model_huge_reddit/model "
            "-t convai2 "
            "--model transformer/crossencoder --batchsize 16 --eval-batchsize 10 "
            "--warmup_updates 1000 --lr-scheduler-patience 0 --lr-scheduler-decay 0.4 "
            "-lr 5e-05 --data-parallel True --history-size 20 --label-truncate 72 "
            "--text-truncate 360 --num-epochs 12.0 --max_train_time 200000 -veps 0.5 "
            "-vme 2500 --validation-metric accuracy --validation-metric-mode max "
            "--save-after-valid True --log_every_n_secs 20 --candidates inline --fp16 True "
            "--dict-tokenizer bpe --dict-lower True --optimizer adamax --output-scaling 0.06 "
            "--variant xlm --reduction-type first --share-encoders False "
            "--learn-positional-embeddings True --n-layers 12 --n-heads 12 --ffn-size 3072 "
            "--attention-dropout 0.1 --relu-dropout 0.0 --dropout 0.1 --n-positions 1024 "
            "--embedding-size 768 --activation gelu --embeddings-scale False --n-segments 2 "
            "--learn-embeddings True --dict-endtoken __start__ "
            "--model-file <YOUR MODEL FILE>"
        ),
        "result": (
            "(subject to some variance, you may see the following as a result of validation of the model)\n"
            "{'exs': 7801, 'accuracy': 0.903 ...}"
        ),
    },
    {
        "title": "Cross-Encoder Transformer Wikipedia/Toronto Books Pretrained Model",
        "id": "pretrained_transformers",
        "path": "zoo:pretrained_transformers/cross_model_huge_wikito",
        "agent": "transformer/crossencoder",
        "task": "pretrained_transformers",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/polyencoder/",
        "description": (
            "Cross-Encoder pretrained on Wikipedia/Toronto Books. Use this model as an ``--init-model`` for a poly-encoder "
            "when fine-tuning on another task. For more details on how to train, see the project page."
        ),
        "example": (
            "parlai train_model "
            "--init-model zoo:pretrained_transformers/cross_model_huge_wikito/model "
            "-t convai2 "
            "--model transformer/crossencoder --batchsize 16 --eval-batchsize 10 "
            "--warmup_updates 1000 --lr-scheduler-patience 0 --lr-scheduler-decay 0.4 "
            "-lr 5e-05 --data-parallel True --history-size 20 --label-truncate 72 "
            "--text-truncate 360 --num-epochs 12.0 --max_train_time 200000 -veps 0.5 "
            "-vme 2500 --validation-metric accuracy --validation-metric-mode max "
            "--save-after-valid True --log_every_n_secs 20 --candidates inline --fp16 True "
            "--dict-tokenizer bpe --dict-lower True --optimizer adamax --output-scaling 0.06 "
            "--variant xlm --reduction-type first --share-encoders False "
            "--learn-positional-embeddings True --n-layers 12 --n-heads 12 --ffn-size 3072 "
            "--attention-dropout 0.1 --relu-dropout 0.0 --dropout 0.1 --n-positions 1024 "
            "--embedding-size 768 --activation gelu --embeddings-scale False --n-segments 2 "
            "--learn-embeddings True --dict-endtoken __start__ "
            "--model-file <YOUR MODEL FILE>"
        ),
        "result": (
            "(subject to some variance, you may see the following as a result of validation of the model)\n"
            "{'exs': 7801, 'accuracy': 0.873 ...}"
        ),
    },
    {
        "title": "Poly-Encoder Transformer ConvAI2 Model",
        "id": "pretrained_transformers",
        "path": "zoo:pretrained_transformers/model_poly",
        "agent": "transformer/polyencoder",
        "task": "convai2",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/polyencoder/",
        "description": (
            "Polyencoder pretrained on Reddit and fine-tuned on ConvAI2 scoring 89+ hits @ 1/20. See the pretrained_transformers directory for a list of other available pretrained transformers"
        ),
        "example": (
            "parlai interactive -mf "
            "zoo:pretrained_transformers/model_poly/model -t convai2"
        ),
        "result": (
            "hi how are you doing ?\n"
            "[Polyencoder]: i am alright . i am back from the library .\n"
            "Enter Your Message: oh, what do you do for a living?\n"
            "[Polyencoder]: i work at the museum downtown . i love it there .\n"
            "Enter Your Message: what is your favorite drink?\n"
            "[Polyencoder]: i am more of a tea guy . i get my tea from china .\n"
        ),
        "example2": (
            "parlai eval_model -mf zoo:pretrained_transformers/model_poly/model -t convai2 --eval-candidates inline"
        ),
        "result2": (
            "[ Finished evaluating tasks ['convai2'] using datatype valid ]\n"
            "{'exs': 7801, 'accuracy': 0.8942, 'f1': 0.9065, 'hits@1': 0.894, 'hits@5': 0.99, 'hits@10': 0.997, 'hits@100': 1.0, 'bleu': 0.8941, 'lr': 5e-09, 'total_train_updates': 0, 'examples': 7801, 'loss': 3004.0, 'mean_loss': 0.385, 'mean_rank': 1.234, 'mrr': 0.9359}"
        ),
    },
    {
        "title": "Bi-Encoder Transformer ConvAI2 Model",
        "id": "pretrained_transformers",
        "path": "zoo:pretrained_transformers/model_bi",
        "agent": "transformer/biencoder",
        "task": "convai2",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/polyencoder/",
        "description": (
            "Bi-encoder pretrained on Reddit and fine-tuned on ConvAI2 scoring ~87 hits @ 1/20."
        ),
        "example": (
            "parlai interactive -mf "
            "zoo:pretrained_transformers/model_bi/model -t convai2"
        ),
        "result": (
            "hi how are you doing ?\n"
            "[Biencoder]: my mother is from russia .\n"
            "Enter Your Message: oh cool, whereabouts ?\n"
            "[Biencoder]: no , she passed away when i was 18 . thinking about russian recipes she taught me ,\n"
            "Enter Your Message: what do you cook?\n"
            "[Biencoder]: like meat mostly , me and my dogs love them , do you like dogs ?\n"
        ),
        "example2": (
            "parlai eval_model -mf zoo:pretrained_transformers/model_bi/model -t convai2 --eval-candidates inline"
        ),
        "result2": (
            "[ Finished evaluating tasks ['convai2'] using datatype valid ]\n"
            "{'exs': 7801, 'accuracy': 0.8686, 'f1': 0.8833, 'hits@1': 0.869, 'hits@5': 0.987, 'hits@10': 0.996, 'hits@100': 1.0, 'bleu': 0.8685, 'lr': 5e-09, 'total_train_updates': 0, 'examples': 7801, 'loss': 28.77, 'mean_loss': 0.003688, 'mean_rank': 1.301, 'mrr': 0.9197}"
        ),
    },
    {
        "title": "TransResNet (ResNet152) Image-Chat model",
        "id": "image_chat",
        "path": "zoo:image_chat/transresnet_multimodal",
        "agent": "projects.image_chat.transresnet_multimodal.transresnet_multimodal:TransresnetMultimodalAgent",  # noqa: E501
        "task": "image_chat",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/image_chat",
        "description": (
            "Transresnet Multimodal Model pretrained on the Image-Chat task"
        ),
        "example": (
            "parlai eval_model -t image_chat "
            "-mf zoo:image_chat/transresnet_multimodal/model -dt test"
        ),
        "result": "{'exs': 29991, 'accuracy': 0.4032, 'f1': 0.4432, 'hits@1': 0.403, 'hits@5': 0.672, 'hits@10': 0.779, 'hits@100': 1.0, 'bleu': 0.3923,"  # noqa: E501
        "'first_round': {'hits@1/100': 0.3392, 'loss': -0.002001, 'med_rank': 3.0},"
        "'second_round': {'hits@1/100': 0.4558, 'loss': -0.002001, 'med_rank': 2.0},"
        "'third_round+': {'hits@1/100': 0.4147, 'loss': -0.002001, 'med_rank': 2.0}}"  # noqa: E501
        "'hits@10': 0.903, 'hits@100': 0.998, 'bleu': 0.4999, 'hits@1/100': 1.0, 'loss': -0.002, 'med_rank': 1.0}",  # noqa: E501
    },
    {
        "title": "Self-feeding Chatbot",
        "id": "self_feeding",
        "path": "zoo:self_feeding/model",
        "agent": "projects.self_feeding.self_feeding_agent:SelfFeedingAgent",
        "task": "self_feeding:all:train",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/self_feeding",
        "description": (
            "The self-feeding chatbot of Hancock, et al., 2019 "
            "(https://arxiv.org/abs/1901.05415). This model learns from is mistakes "
            "when actually talking with users. This particular model corresponds to "
            "the model with 131k human-human chats + 60k human-bot chats + 60k "
            "feedback chats."
        ),
        "example": (
            "python projects/self_feeding/interactive.py --model-file "
            "zoo:self_feeding/hh131k_hb60k_fb60k_st1k/model --no-cuda true"
        ),
        "result": (
            "Enter Your Message: hi, my name is stephen. what's yours?\n"
            "[SelfFeeding]: hi there greg . do you have pets ? i've 2 cats named "
            "milo and fio .\n"
            "Enter Your Message: sadly, i have no pets. my landlord isn't a fan.\n"
            "[SelfFeeding]: sorry to hear that . i always had bad allergies when i "
            "liven on my farm in kansas ."
        ),
        "example2": (
            "parlai eval_model -mf "
            "zoo:self_feeding/hh131k_hb60k_fb60k_st1k/model -t self_feeding:all"
        ),
        "result2": (
            "[ Finished evaluating tasks ['self_feeding:all'] using datatype valid ]\n"
            "{'exs': 3500, 'dia_rank': 4.654, 'dia_acc': 0.3525, 'fee_rank': 1.0, "
            "'fee_acc': 1.0, 'fee_exs': 1000, 'sat_re': 0.4607, 'sat_f1': 0.5605, "
            "'sat_acc': 0.724}"
        ),
    },
    {
        "title": "Transformer Classifier Single-turn Dialogue Safety Model",
        "id": "dialogue_safety",
        "path": "zoo:dialogue_safety/single_turn/model",
        "agent": "transformer/classifier",
        "task": "dialogue_safety:adversarial,dialogue_safety:standard",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dialogue_safety",
        "description": (
            "Classifier trained on both the standard and adversarial safety tasks in addition to Wikipedia Toxic Comments."
        ),
        "example": (
            "parlai eval_model -t dialogue_safety:adversarial "
            "--round 3 -dt test -mf zoo:dialogue_safety/single_turn/model -bs 40"
        ),
        "result": (
            "{'exs': 3000, 'accuracy': 0.9627, 'f1': 0.9627, 'bleu': 9.627e-10, 'lr': 5e-09, 'total_train_updates': 0, 'examples': 3000, 'mean_loss': 0.005441, 'class___notok___recall': 0.7833, 'class___notok___prec': 0.8333, 'class___notok___f1': 0.8076, 'class___ok___recall': 0.9826, 'class___ok___prec': 0.9761, 'class___ok___f1': 0.9793, 'weighted_f1': 0.9621}"
        ),
    },
    {
        "title": "BERT Classifier Multi-turn Dialogue Safety Model",
        "id": "dialogue_safety",
        "path": "zoo:dialogue_safety/multi_turn/model",
        "agent": "bert_classifier",
        "task": "dialogue_safety:multiturn",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dialogue_safety",
        "description": (
            "Classifier trained on the multi-turn adversarial safety task in addition to both the single-turn standard and adversarial safety tasks and Wikipedia Toxic Comments."
        ),
        "example": (
            "parlai eval_model -t dialogue_safety:multiturn -dt test -mf zoo:dialogue_safety/multi_turn/model --split-lines True -bs 40"
        ),
        "result": (
            "{'exs': 3000, 'accuracy': 0.9317, 'f1': 0.9317, 'bleu': 9.317e-10, 'lr': 5e-09, 'total_train_updates': 0, 'examples': 3000, 'mean_loss': 0.008921, 'class___notok___recall': 0.7067, 'class___notok___prec': 0.6444, 'class___notok___f1': 0.6741, 'class___ok___recall': 0.9567, 'class___ok___prec': 0.9671, 'class___ok___f1': 0.9618, 'weighted_f1': 0.9331}"
        ),
    },
    {
        "title": "Integration Test Models",
        "id": "unittest",
        "path": "zoo:unittest/transformer_ranker/model",
        "task": "integration_tests",
        "description": (
            "Model files used to check backwards compatibility and code coverage of important standard models."
        ),
        "example": (
            "parlai eval_model -mf zoo:unittest/transformer_generator2/model -t integration_tests:multiturn_candidate -m transformer/generator"
        ),
        "external_website": '',
        "result": (
            """{'exs': 400, 'accuracy': 1.0, 'f1': 1.0, 'bleu-4': 0.2503, 'lr': 0.001, 'total_train_updates': 5000, 'gpu_mem_percent': 9.37e-05, 'loss': 0.0262, 'token_acc': 1.0, 'nll_loss': 7.935e-05, 'ppl': 1.0}"""
        ),
    },
    {
        "title": "ImageSeq2Seq DodecaDialogue All Tasks MT Model",
        "id": "dodecadialogue",
        "path": "zoo:dodecadialogue/all_tasks_mt/model",
        "agent": "image_seq2seq",
        "task": "#Dodeca",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dodecadialogue/",
        "description": "Image Seq2Seq model trained on all DodecaDialogue tasks",
        "example": (
            "parlai interactive -mf zoo:dodecadialogue/all_tasks_mt/model "
            "--inference beam --beam-size 3 --beam-min-length 10 --beam-block-ngram 3 --beam-context-block-ngram 3"
        ),
        "result": (
            "Enter Your Message: hi how are you?\n"
            "[ImageSeq2seq]: i ' m doing well . how are you ?\n"
            "Enter Your Message: not much, what do you like to do?\n"
            "[ImageSeq2seq]: i like to go to the park and play with my friends ."
        ),
        "example2": (
            "parlai eval_model -mf zoo:dodecadialogue/all_tasks_mt/model -t \"#Dodeca\""
            "--prepend-personality True --prepend-gold-knowledge True --image-mode no_image_model"
        ),
        "result2": (
            "[ Finished evaluating tasks ['#Dodeca'] using datatype valid ]\n"
            "                           exs  gpu_mem  loss        lr   ppl  token_acc  total_train_updates  tpb\n"
            "   WizTeacher             3939          2.161           8.678      .5325\n"
            "   all                   91526    .3371 2.807 9.375e-07 18.23      .4352               470274 2237\n"
            "   convai2                7801          2.421           11.26      .4721\n"
            "   cornell_movie         13905          3.088           21.93      .4172\n"
            "   dailydialog            8069           2.47           11.82      .4745\n"
            "   empathetic_dialogues   5738          2.414           11.18      .4505\n"
            "   igc                     486          2.619           13.73      .4718\n"
            "   image_chat:Generation 15000          3.195           24.42      .3724\n"
            "   light_dialog           6623          2.944              19      .3918\n"
            "   twitter               10405           3.61           36.98      .3656\n"
            "   ubuntu                19560          3.148            23.3      .4035"
        ),
    },
    {
        "title": "ImageSeq2Seq DodecaDialogue ConvAI2 FT Model",
        "id": "dodecadialogue",
        "path": "zoo:dodecadialogue/convai2_ft/model",
        "agent": "image_seq2seq",
        "task": "convai2",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dodecadialogue/",
        "description": (
            "Image Seq2Seq model trained on all DodecaDialogue tasks and fine-tuned on Convai2"
        ),
        "example": (
            "parlai interactive -mf zoo:dodecadialogue/convai2_ft/model -t convai2 "
            "--inference beam --beam-size 3 --beam-min-length 10 --beam-block-ngram 3 --beam-context-block-ngram 3"
        ),
        "result": (
            "[context]: your persona: i currently work for ibm in chicago.\n"
            "your persona: i'm not a basketball player though.\n"
            "your persona: i am almost 7 feet tall.\n"
            "your persona: i'd like to retire to hawaii in the next 10 years.\n"
            "Enter Your Message: hi how's it going\n"
            "[ImageSeq2seq]: i ' m doing well . how are you ?\n"
            "Enter Your Message: i'm well, i am really tall\n"
            "[ImageSeq2seq]: that ' s cool . i like simple jokes ."
        ),
        "example2": (
            "parlai eval_model -mf zoo:dodecadialogue/convai2_ft/model -t convai2"
        ),
        "result2": (
            "[ Finished evaluating tasks ['convai2'] using datatype valid ]\n"
            "    exs  gpu_mem  loss      lr   ppl  token_acc  total_train_updates   tpb\n"
            "   7801    .2993 2.415 7.5e-06 11.19      .4741                15815 845.8"
        ),
    },
    {
        "title": "ImageSeq2Seq DodecaDialogue Cornell Movie FT Model",
        "id": "dodecadialogue",
        "path": "zoo:dodecadialogue/cornell_movie_ft/model",
        "agent": "image_seq2seq",
        "task": "cornell_movie",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dodecadialogue/",
        "description": (
            "Image Seq2Seq model trained on all DodecaDialogue tasks and fine-tuned on the Cornell Movie task"
        ),
        "example": (
            "parlai interactive -mf zoo:dodecadialogue/cornell_movie_ft/model "
            "--inference beam --beam-size 10 --beam-min-length 20 --beam-block-ngram 3 --beam-context-block-ngram 3"
        ),
        "result": (
            "Enter Your Message: hi how's it going?\n"
            "[ImageSeq2seq]: oh , it ' s great . i ' m having a great time . how are you doing ?\n"
            "Enter Your Message: i'm doing well, what do you like to do?\n"
            "[ImageSeq2seq]: i like to go to the movies . what about you ? do you have any hobbies ?"
        ),
        "example2": (
            "parlai eval_model -mf zoo:dodecadialogue/cornell_movie_ft/model -t cornell_movie"
        ),
        "result2": (
            "[ Finished evaluating tasks ['cornell_movie'] using datatype valid ]\n"
            "     exs  gpu_mem  loss      lr   ppl  token_acc  total_train_updates   tpb\n"
            "   13905   .07094 2.967 2.5e-06 19.43      .4290                29496 15.76"
        ),
    },
    {
        "title": "ImageSeq2Seq DodecaDialogue DailyDialog FT Model",
        "id": "dodecadialogue",
        "path": "zoo:dodecadialogue/daily_dialog_ft/model",
        "agent": "image_seq2seq",
        "task": "dailydialog",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dodecadialogue/",
        "description": (
            "Image Seq2Seq model trained on all DodecaDialogue tasks and fine-tuned on the DailyDialog task"
        ),
        "example": (
            "parlai interactive -mf zoo:dodecadialogue/daily_dialog_ft/model "
            "--inference beam --beam-size 5 --beam-min-length 10 --beam-block-ngram 3 --beam-context-block-ngram 3"
        ),
        "result": (
            "Enter Your Message: hi how's it going\n"
            "[ImageSeq2seq]: i ' m doing well . how about you ?\n"
            "Enter Your Message: not much, what do you like to do?\n"
            "[ImageSeq2seq]: i like to go to the beach and play volleyball ."
        ),
        "example2": (
            "parlai eval_model -mf zoo:dodecadialogue/daily_dialog_ft/model -t dailydialog"
        ),
        "result2": (
            "[ Finished evaluating tasks ['dailydialog'] using datatype valid ]\n"
            "    exs  gpu_mem  loss      lr   ppl  token_acc  total_train_updates   tpb\n"
            "   8069   .06787 2.326 7.5e-06 10.24      .5093               150959 15.67"
        ),
    },
    {
        "title": "ImageSeq2Seq DodecaDialogue ELI5 FT Model",
        "id": "dodecadialogue",
        "path": "zoo:dodecadialogue/eli5_ft/model",
        "agent": "image_seq2seq",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dodecadialogue/",
        "task": "TBD",
        "description": (
            "Image Seq2Seq model trained on all DodecaDialogue tasks and fine-tuned on the ELI5 task. "
            "The ELI5 model in the zoo is the one that scored the highest ROUGE scores; "
            "it may have slightly lower f1, bleu, and slightly higher ppl than the numbers "
            "reported in the dodeca paper."
        ),
        "example": (
            "parlai interactive -mf zoo:dodecadialogue/eli5_ft/model "
            "--inference beam --beam-size 10 --beam-min-length 200 --beam-block-ngram 3 --beam-context-block-ngram 3"
        ),
        "result": (
            "Enter Your Message: Hi, can you tell me about quantum physics?\n"
            "[ImageSeq2seq]: yes , i can . quantum physics is the study of how particles "
            "interact with each other , and how they interact with other particles . "
            "it ' s important to note that quantum mechanics is n ' t the same as "
            "classical physics . classical physics is a study of the properties of "
            "particles , and what they do . in classical physics , there are two "
            "types of particles : quarks and neutrinos . quarks are made up of quarks , "
            "neutrinos , and electrons . neutrinos are made of protons , neutrons , "
            "electrons , and neutrons . they ' re all the same thing , but they all "
            "have the same properties . so , if you ' re interested in quantum physics , "
            "you might want to check out / r / askscience . there ' s a subreddit "
            "for that sort of thing . edit : i ' m not sure what you mean by "
            "\" quantum physics \" , but i ' ll let you know if you want to know more . "
            "edit 2 : thanks for the gold !"
        ),
    },
    {
        "title": "ImageSeq2Seq DodecaDialogue Empathetic Dialogue FT Model",
        "id": "dodecadialogue",
        "path": "zoo:dodecadialogue/empathetic_dialogues_ft/model",
        "agent": "image_seq2seq",
        "task": "empathetic_dialogues",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dodecadialogue/",
        "description": (
            "Image Seq2Seq model trained on all DodecaDialogue tasks and fine-tuned on the Empathetic Dialogue task"
        ),
        "example": (
            "parlai interactive -mf zoo:dodecadialogue/empathetic_dialogues_ft/model "
            "--inference beam --beam-size 5 --beam-min-length 10 --beam-block-ngram 3 --beam-context-block-ngram 3"
        ),
        "result": (
            "Enter Your Message: hi, how's it going?\n"
            "[ImageSeq2seq]: i ' m doing well . how are you ?\n"
            "Enter Your Message: i'm fine, feeling a little sad\n"
            "[ImageSeq2seq]: that ' s too bad . what ' s going on ?"
        ),
        "example2": (
            "parlai eval_model -mf zoo:dodecadialogue/empathetic_dialogues_ft/model -t empathetic_dialogues"
        ),
        "result2": (
            "[ Finished evaluating tasks ['empathetic_dialogues'] using datatype valid ]\n"
            "    exs  gpu_mem  loss      lr   ppl  token_acc  total_train_updates  tpb\n"
            "   5738    .3278 2.405 7.5e-06 11.08      .4517                20107 1914"
        ),
    },
    {
        "title": "ImageSeq2Seq DodecaDialogue Image Grounded Conversations FT Model",
        "id": "dodecadialogue",
        "path": "zoo:dodecadialogue/igc_ft/model",
        "agent": "image_seq2seq",
        "task": "igc",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dodecadialogue/",
        "description": (
            "Image Seq2Seq model trained on all DodecaDialogue tasks and fine-tuned on the Image Grounded Conversations task"
        ),
        "example": (
            "parlai eval_model -mf zoo:dodecadialogue/igc_ft/model -t igc:responseOnly"
        ),
        "result": (
            "[ Finished evaluating tasks ['igc:responseOnly'] using datatype valid ]\n"
            "    exs  gpu_mem  loss    lr   ppl  token_acc  total_train_updates   tpb\n"
            "    162    .0726 2.832 1e-06 16.98      .4405                10215 9.852"
        ),
    },
    {
        "title": "ImageSeq2Seq DodecaDialogue Image Chat FT Model",
        "id": "dodecadialogue",
        "path": "zoo:dodecadialogue/image_chat_ft/model",
        "agent": "image_seq2seq",
        "task": "image_chat",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dodecadialogue/",
        "description": (
            "Image Seq2Seq model trained on all DodecaDialogue tasks and fine-tuned on the Image Chat task"
        ),
        "example": (
            "parlai eval_model -mf zoo:dodecadialogue/image_chat_ft/model -t image_chat:generation "
            "--image-mode no_image_model"
        ),
        "result": (
            "[ Finished evaluating tasks ['image_chat:generation'] using datatype valid ]\n"
            "     exs  gpu_mem  loss        lr   ppl  token_acc  total_train_updates  tpb\n"
            "   15000    .2231 4.353 3.125e-07 77.73      .2905               321001 1653"
        ),
    },
    {
        "title": "ImageSeq2Seq DodecaDialogue LIGHT Dialogue FT Model",
        "id": "dodecadialogue",
        "path": "zoo:dodecadialogue/light_dialog_ft/model",
        "agent": "image_seq2seq",
        "task": "light_dialog",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dodecadialogue/",
        "description": (
            "Image Seq2Seq model trained on all DodecaDialogue tasks and fine-tuned on the LIGHT Dialogue task"
        ),
        "example": (
            "parlai interactive -mf zoo:dodecadialogue/light_dialog_ft/model "
            "--inference beam --beam-size 5 --beam-min-length 20 --beam-block-ngram 3 --beam-context-block-ngram 3"
        ),
        "result": (
            "Enter Your Message: hi how's it going?\n"
            "[ImageSeq2seq]: i ' m doing well . how about you ? what ' s going on in the world today ?\n"
            "Enter Your Message: not much, wish it had some more epic battles!\n"
            "[ImageSeq2seq]: me too . it ' s been so long since i ' ve seen a battle like this . do you have a favorite battle ?"
        ),
        "example2": (
            "parlai eval_model -mf zoo:dodecadialogue/light_dialog_ft/model -t light_dialog"
        ),
        "result2": (
            "[ Finished evaluating tasks ['light_dialog'] using datatype valid ]\n"
            "    exs  gpu_mem  loss      lr   ppl  token_acc  total_train_updates   tpb\n"
            "   6623   .07002 2.927 7.5e-06 18.66      .3927                38068 20.81"
        ),
    },
    {
        "title": "ImageSeq2Seq DodecaDialogue pushshift.io Reddit FT Model",
        "id": "dodecadialogue",
        "path": "zoo:dodecadialogue/reddit_ft/model",
        "agent": "image_seq2seq",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dodecadialogue/",
        "task": "TBD",
        "description": (
            "Image Seq2Seq model trained on all DodecaDialogue tasks and fine-tuned on the pushshift.io Reddit task"
        ),
        "example": (
            "parlai interactive -mf zoo:dodecadialogue/reddit_ft/model "
            "--inference beam --beam-size 5 --beam-min-length 20 --beam-block-ngram 3 --beam-context-block-ngram 3"
        ),
        "result": (
            "Enter Your Message: hi how's it going?\n"
            "[ImageSeq2seq]: hi , i ' m doing pretty well . how are you ? : ) and yourself ? : d\n"
            "Enter Your Message: just hanging in there, you up to anything fun?\n"
            "[ImageSeq2seq]: not really . i just got home from work . i ' ll be back in a few hours ."
        ),
    },
    {
        "title": "ImageSeq2Seq DodecaDialogue Twitter FT Model",
        "id": "dodecadialogue",
        "path": "zoo:dodecadialogue/twitter_ft/model",
        "agent": "image_seq2seq",
        "task": "twitter",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dodecadialogue/",
        "description": (
            "Image Seq2Seq model trained on all DodecaDialogue tasks and fine-tuned on the Twitter task"
        ),
        "example": (
            "parlai interactive -mf zoo:dodecadialogue/twitter_ft/model "
            "--inference beam --beam-size 10 --beam-min-length 20 --beam-block-ngram 3 --beam-context-block-ngram 3"
        ),
        "result": (
            "Enter Your Message: hi how's it going?\n"
            "[ImageSeq2seq]: it ' s going well ! how are you ? @ smiling_face_with_heart - eyes @\n"
            "Enter Your Message: im doing well, what do you like to do\n"
            "[ImageSeq2seq]: hi ! i ' m doing well ! i like to read , watch movies , play video games , and listen to music . how about you ?"
        ),
        "example2": (
            "parlai eval_model -mf zoo:dodecadialogue/twitter_ft/model -t twitter"
        ),
        "result2": (
            "[ Finished evaluating tasks ['twitter'] using datatype valid ]\n"
            "     exs  gpu_mem  loss      lr   ppl  token_acc  total_train_updates  tpb\n"
            "   10405    .3807 3.396 7.5e-06 29.83      .3883               524029 2395"
        ),
    },
    {
        "title": "ImageSeq2Seq DodecaDialogue Ubuntu V2 FT Model",
        "id": "dodecadialogue",
        "path": "zoo:dodecadialogue/ubuntu_ft/model",
        "agent": "image_seq2seq",
        "task": "ubuntu",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dodecadialogue/",
        "description": (
            "Image Seq2Seq model trained on all DodecaDialogue tasks and fine-tuned on the Ubuntu V2 task"
        ),
        "example": (
            "parlai interactive -mf zoo:dodecadialogue/ubuntu_ft/model "
            "--inference beam --beam-size 2 --beam-min-length 10 --beam-block-ngram 3 --beam-context-block-ngram 3"
        ),
        "result": (
            "Enter Your Message: hi how's it going?\n"
            "[ImageSeq2seq]: i ' m fine . . . you ? .\n"
            "Enter Your Message: doing ok, what do you like to do?\n"
            "[ImageSeq2seq]: i like to read , write , and read ."
        ),
        "example2": (
            "parlai eval_model -mf zoo:dodecadialogue/ubuntu_ft/model -t ubuntu"
        ),
        "result2": (
            "[ Finished evaluating tasks ['ubuntu'] using datatype valid ]\n"
            "     exs  gpu_mem  loss      lr   ppl  token_acc  total_train_updates  tpb\n"
            "   19560    .3833 2.844 2.5e-05 17.18      .4389               188076 3130"
        ),
    },
    {
        "title": "ImageSeq2Seq DodecaDialogue Wizard of Wikipedia FT Model",
        "id": "dodecadialogue",
        "path": "zoo:dodecadialogue/wizard_of_wikipedia_ft/model",
        "agent": "image_seq2seq",
        "task": "wizard_of_wikipedia:Generator",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dodecadialogue/",
        "description": (
            "Image Seq2Seq model trained on all DodecaDialogue tasks and fine-tuned on the Wizard of Wikipedia task"
        ),
        "example": (
            "parlai interactive -mf zoo:dodecadialogue/wizard_of_wikipedia_ft/model "
            "--inference beam --beam-size 10 --beam-min-length 10 --beam-block-ngram 3 --beam-context-block-ngram 3 -t wizard_of_wikipedia"
        ),
        "result": (
            "Please choose one of the following topics by typing A, B, C, ..., etc. :\n"
            "\n"
            "A: Denver Art Museum\n"
            "B: Joke\n"
            "C: Sport utility vehicle\n"
            "D: Chocolate brownie\n"
            "E: [NO TOPIC]\n"
            "\n"
            "Enter Your Message: D\n"
            "[ Your chosen topic is: Chocolate brownie ]\n"
            "/private/home/kshuster/ParlAI/parlai/core/torch_ranker_agent.py:670: UserWarning: [ Executing eval mode with provided inline set of candidates ]\n"
            "  ''.format(mode)\n"
            "[ Using chosen sentence from Wikpedia ]: A chocolate brownie (commonly referred to as simply brownie) is a square, baked, chocolate dessert.\n"
            "[ImageSeq2seq]: i love chocolate brownies . they ' re my favorite .\n"
            "Enter Your Message: me too! do you ever make them?\n"
            "[ Using chosen sentence from Wikpedia ]: Brownies are typically eaten by hand, often accompanied by milk, served warm with ice cream (a la mode), topped with whipped cream, or sprinkled with powdered sugar and fudge.\n"
            "[ImageSeq2seq]: i don ' t make them , but i would love to ."
        ),
        "example2": (
            "parlai eval_model -mf zoo:dodecadialogue/wizard_of_wikipedia_ft/model -t wizard_of_wikipedia:Generator --prepend-gold-knowledge true"
        ),
        "result2": (
            "[ Finished evaluating tasks ['wizard_of_wikipedia:Generator'] using datatype valid ]\n"
            "    exs  gpu_mem  loss      lr   ppl  token_acc  total_train_updates  tpb\n"
            "   3939    .3823 2.144 7.5e-06 8.532      .5348                22908 2852"
        ),
    },
    {
        "title": "ImageSeq2Seq DodecaDialogue Base Model",
        "id": "dodecadialogue",
        "path": "zoo:dodecadialogue/base_model/model",
        "agent": "image_seq2seq",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dodecadialogue/",
        "task": "#Dodeca",
        "description": (
            "Image Seq2Seq base model, from which all DodecaDialogue models were trained"
        ),
        "example": (
            "parlai train_model -t \"#Dodeca\" --prepend-gold-knowledge true --prepend-personality true -mf /tmp/dodeca_model --init-model zoo:dodecadialogue/base_model/model --dict-file zoo:dodecadialogue/dict/dodeca.dict --model image_seq2seq --dict-tokenizer bpe --dict-lower true -bs 32 -eps 0.5 -esz 512 --ffn-size 2048 --fp16 false --n-heads 16 --n-layers 8 --n-positions 512 --text-truncate 512 --label-truncate 128 --variant xlm -lr 7e-6 --lr-scheduler reduceonplateau --optimizer adamax --dropout 0.1 --validation-every-n-secs 3600 --validation-metric ppl --validation-metric-mode min --validation-patience 10 --activation gelu --embeddings-scale true --learn-positional-embeddings true --betas 0.9,0.999 --warmup-updates 2000 --gradient-clip 0.1"
        ),
        "result": "A trained model (logs omitted)",
    },
    {
        "title": "BlendedSkillTalk: BlendedSkillTalk single-task model",
        "id": "blended_skill_talk",
        "path": "zoo:blended_skill_talk/bst_single_task/model",
        "agent": "transformer/polyencoder",
        "task": "blended_skill_talk",
        "project": 'https://github.com/facebookresearch/ParlAI/tree/master/projects/bst',
        "description": "Pretrained polyencoder retrieval model fine-tuned on the BlendedSkillTalk dialogue task.",
        "example": "parlai interactive -mf zoo:blended_skill_talk/bst_single_task/model -t blended_skill_talk",
        "result": 'Results vary.',
        "example2": "parlai eval_model -mf zoo:blended_skill_talk/bst_single_task/model -t blended_skill_talk -dt test",
        "result2": """09:51:57 | Finished evaluating tasks ['blended_skill_talk'] using datatype test
    accuracy  bleu-4  exs    f1  gpu_mem  hits@1  hits@10  hits@100  hits@5  loss   mrr  rank   tpb
       .7920   .7785 5482 .8124    .0370   .7920    .9788         1   .9542 .8251 .8636 1.866 19.76
""",
    },
    {
        "title": "BlendedSkillTalk: ConvAI2 single-task model",
        "id": "blended_skill_talk",
        "path": "zoo:blended_skill_talk/convai2_single_task/model",
        "agent": "transformer/polyencoder",
        "task": "blended_skill_talk",
        "project": 'https://github.com/facebookresearch/ParlAI/tree/master/projects/bst',
        "description": "Pretrained polyencoder retrieval model fine-tuned on the ConvAI2 dialogue task.",
        "example": "parlai eval_model -mf zoo:blended_skill_talk/convai2_single_task/model -t blended_skill_talk -dt test",
        "result": """10:23:53 | Finished evaluating tasks ['blended_skill_talk'] using datatype test
    accuracy  bleu-4  exs    f1  gpu_mem  hits@1  hits@10  hits@100  hits@5  loss   mrr  rank   tpb
       .7678   .7553 5482 .7902   .07928   .7678    .9728         1   .9414 .9337 .8451  2.04 19.76
""",
    },
    {
        "title": "BlendedSkillTalk: EmpatheticDialogues single-task model",
        "id": "blended_skill_talk",
        "path": "zoo:blended_skill_talk/ed_single_task/model",
        "agent": "transformer/polyencoder",
        "task": "blended_skill_talk",
        "project": 'https://github.com/facebookresearch/ParlAI/tree/master/projects/bst',
        "description": "Pretrained polyencoder retrieval model fine-tuned on the EmpatheticDialogues dialogue task.",
        "example": "parlai eval_model -mf zoo:blended_skill_talk/ed_single_task/model -t blended_skill_talk -dt test",
        "result": """10:16:47 | Finished evaluating tasks ['blended_skill_talk'] using datatype test
    accuracy  bleu-4  exs    f1  gpu_mem  hits@1  hits@10  hits@100  hits@5  loss   mrr  rank   tpb
       .6895   .6774 5482 .7219   .07928   .6895    .9509         1   .9051 1.242 .7849  2.79 19.76
""",
    },
    {
        "title": "BlendedSkillTalk: Wizard of Wikipedia single-task model",
        "id": "blended_skill_talk",
        "path": "zoo:blended_skill_talk/wizard_single_task/model",
        "agent": "transformer/polyencoder",
        "task": "blended_skill_talk",
        "project": 'https://github.com/facebookresearch/ParlAI/tree/master/projects/bst',
        "description": "Pretrained polyencoder retrieval model fine-tuned on the Wizard of Wikipedia dialogue task.",
        "example": "parlai eval_model -mf zoo:blended_skill_talk/wizard_single_task/model -t blended_skill_talk -dt test",
        "result": """10:34:46 | Finished evaluating tasks ['blended_skill_talk'] using datatype test
    accuracy  bleu-4  exs    f1  gpu_mem  hits@1  hits@10  hits@100  hits@5  loss   mrr  rank   tpb
       .6742   .6616 5482 .7059   .07928   .6742    .9445         1   .8902 1.321 .7706 2.962 19.76
""",
    },
    {
        "title": "BlendedSkillTalk: MT Single-Skills model",
        "id": "blended_skill_talk",
        "path": "zoo:blended_skill_talk/multi_task/model",
        "agent": "transformer/polyencoder",
        "task": "blended_skill_talk",
        "project": 'https://github.com/facebookresearch/ParlAI/tree/master/projects/bst',
        "description": "Pretrained polyencoder retrieval model fine-tuned on the ConvAI2, EmpatheticDialogues, and Wizard of Wikipedia dialogue tasks.",
        "example": "parlai eval_model -mf zoo:blended_skill_talk/multi_task/model -t blended_skill_talk -dt test",
        "result": """10:23:35 | Finished evaluating tasks ['blended_skill_talk'] using datatype test
    accuracy  bleu-4  exs    f1  gpu_mem  hits@1  hits@10  hits@100  hits@5  loss   mrr  rank   tpb
       .8010   .7872 5482 .8204   .07928   .8010    .9779         1   .9564 .8154 .8697 1.908 19.76
""",
    },
    {
        "title": "BlendedSkillTalk: MT Single-Skills model fine-tuned on BST",
        "id": "blended_skill_talk",
        "path": "zoo:blended_skill_talk/multi_task_bst_tuned/model",
        "agent": "transformer/polyencoder",
        "task": "blended_skill_talk",
        "project": 'https://github.com/facebookresearch/ParlAI/tree/master/projects/bst',
        "description": "Pretrained polyencoder retrieval model fine-tuned on the ConvAI2, EmpatheticDialogues, and Wizard of Wikipedia dialogue tasks, and then further fine-tuned on the BlendedSkillTalk dialogue task.",
        "example": "parlai eval_model -mf zoo:blended_skill_talk/multi_task_bst_tuned/model -t blended_skill_talk -dt test",
        "result": """10:36:01 | Finished evaluating tasks ['blended_skill_talk'] using datatype test
    accuracy  bleu-4  exs    f1  gpu_mem  hits@1  hits@10  hits@100  hits@5  loss   mrr  rank   tpb
       .8378   .8230 5482 .8543   .07928   .8378    .9872         1   .9704 .5897 .8963 1.604 19.76
""",
    },
    {
        "title": "Tutorial Transformer Generator",
        "id": "tutorial_transformer_generator",
        "path": "zoo:tutorial_transformer_generator/model",
        "task": "pushshift.io",
        "description": (
            "Small (87M paramter) generative transformer, pretrained on pushshift.io Reddit."
        ),
        "example": "parlai interactive -mf zoo:tutorial_transformer_generator/model",
        "external_website": '',
        "result": (
            "Enter Your Message: hi, how are you today?\n"
            "[TransformerGenerator]: i ' m doing well , how about you ?\n"
            "Enter Your Message: I'm giving a tutorial on chatbots!\n"
            "[TransformerGenerator]: that ' s awesome ! what ' s it about ?\n"
            "Enter Your Message: bots just like you\n"
            "[TransformerGenerator]: i ' ll be sure to check it out !"
        ),
    },
    {
        "title": "Blender 90M",
        "id": "blender",
        "path": "zoo:blender/blender_90M/model",
        "agent": "transformer/generator",
        "task": "blended_skill_talk",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/blender",
        "description": (
            "90< parameter generative model finetuned on blended_skill_talk tasks."
        ),
        "example": (
            "python parlai/scripts/safe_interactive.py -mf zoo:blender/blender_90M/model -t blended_skill_talk"
        ),
        "result": (
            "Enter Your Message: Hi what's up?\n"
            "[TransformerGenerator]: hello , how are you ? i just got back from working at a law firm , how about you ?"
        ),
    },
    {
        "title": "Blender 2.7B",
        "id": "blender",
        "path": "zoo:blender/blender_3B/model",
        "agent": "transformer/generator",
        "task": "blended_skill_talk",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/blender",
        "description": (
            "2.7B parameter generative model finetuned on blended_skill_talk tasks."
        ),
        "example": (
            "python parlai/scripts/safe_interactive.py -mf zoo:blender/blender_3B/model -t blended_skill_talk"
        ),
        "result": (
            "Enter Your Message: Hi how are you?\n"
            "[TransformerGenerator]: I'm doing well. How are you doing? What do you like to do in your spare time?"
        ),
    },
    {
        "title": "Blender 1B distilled",
        "id": "blender",
        "path": "zoo:blender/blender_1Bdistill/model",
        "agent": "transformer/generator",
        "task": "blended_skill_talk",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/blender",
        "description": (
            "2.7B parameter generative model finetuned on blended_skill_talk tasks and then distilled to ~1.4B parameters and roughly 2x the inference speed."
        ),
        "example": (
            "python parlai/scripts/safe_interactive.py -mf zoo:blender/blender_1Bdistill/model -t blended_skill_talk"
        ),
        "result": (
            "Enter Your Message: Hi how are you?\n"
            "[TransformerGenerator]: I'm doing well. How about yourself? What do you do for a living? I'm a creative writer."
        ),
    },
    {
        "title": "Blender 400M distilled",
        "id": "blender",
        "path": "zoo:blender/blender_400Mdistill/model",
        "agent": "transformer/generator",
        "task": "blended_skill_talk",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/blender",
        "description": (
            "2.7B parameter generative model finetuned on blended_skill_talk tasks and then distilled to ~360M parameters and roughly 5x the inference speed."
        ),
        "example": (
            "python parlai/scripts/safe_interactive.py -mf zoo:blender/blender_400Mdistill/model -t blended_skill_talk"
        ),
        "result": (
            "Enter Your Message: Hi how are you?\n"
            "[TransformerGenerator]: I'm doing well. How about you? What do you like to do in your free time?"
        ),
    },
    {
        "title": "Blender 9.4B",
        "id": "blender",
        "path": "zoo:blender/blender_9B/model",
        "agent": "transformer/generator",
        "task": "blended_skill_talk",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/blender",
        "description": (
            "9.4B parameter generative model finetuned on blended_skill_talk tasks."
        ),
        "example": (
            "python parlai/scripts/safe_interactive.py -mf zoo:blender/blender_9B/model -t blended_skill_talk"
        ),
        "result": (
            "Enter Your Message: Hi!\n"
            "[TransformerGenerator]: What do you do for a living? I'm a student at Miami University."
        ),
    },
    {
        "title": "Reddit 2.7B",
        "id": "blender",
        "path": "zoo:blender/reddit_3B/model",
        "agent": "transformer/generator",
        "task": "pushshift.io",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/blender",
        "description": (
            "2.7B parameter generative model pretrained on Reddit but not finetuned."
        ),
        "example": (
            "parlai train_model -t blended_skill_talk,wizard_of_wikipedia,convai2:normalized,empathetic_dialogues --multitask-weights 1,3,3,3 -veps 0.25 --attention-dropout 0.0 --batchsize 128 --model transformer/generator --embedding-size 2560 --ffn-size 10240 --variant prelayernorm --n-heads 32 --n-positions 128 --n-encoder-layers 2 --n-decoder-layers 24 --history-add-global-end-token end --delimiter '  ' --dict-tokenizer bytelevelbpe  --dropout 0.1 --fp16 True --init-model zoo:blender/reddit_3B/model --dict-file zoo:blender/reddit_3B/model.dict --label-truncate 128 --log_every_n_secs 10 -lr 7e-06 --lr-scheduler reduceonplateau --lr-scheduler-patience 3 --optimizer adam --relu-dropout 0.0 --activation gelu --model-parallel true --save-after-valid True --text-truncate 128 --truncate 128 --warmup_updates 100 --fp16-impl mem_efficient --update-freq 2 --gradient-clip 0.1 --skip-generation True -vp 10 -vmt ppl -vmm min --model-file /tmp/test_train_27B"
        ),
        "result": "Results vary.",
    },
    {
        "title": "Reddit 9.4B",
        "id": "blender",
        "path": "zoo:blender/reddit_9B/model",
        "agent": "transformer/generator",
        "task": "pushshift.io",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/blender",
        "description": (
            "9.4B parameter generative model pretrained on Reddit but not finetuned."
        ),
        "example": (
            "parlai train_model -t blended_skill_talk,wizard_of_wikipedia,convai2:normalized,empathetic_dialogues --multitask-weights 1,3,3,3 -veps 0.25 --attention-dropout 0.0 --batchsize 8 --eval-batchsize 64 --model transformer/generator --embedding-size 4096 --ffn-size 16384 --variant prelayernorm --n-heads 32 --n-positions 128 --n-encoder-layers 4 --n-decoder-layers 32 --history-add-global-end-token end --dict-tokenizer bytelevelbpe --dropout 0.1 --fp16 True --init-model zoo:blender/reddit_9B/model --dict-file zoo:blender/reddit_9B/model.dict --label-truncate 128 -lr 3e-06 -dynb full --lr-scheduler cosine --max-lr-steps 9000 --lr-scheduler-patience 3 --optimizer adam --relu-dropout 0.0 --activation gelu --model-parallel true --save-after-valid False --text-truncate 128 --truncate 128 --warmup_updates 1000 --fp16-impl mem_efficient --update-freq 4 --log-every-n-secs 30 --gradient-clip 0.1 --skip-generation True -vp 10 --max-train-time 84600 -vmt ppl -vmm min --model-file /tmp/test_train_94B"
        ),
        "result": "Results vary.",
    },
    {
        "title": "BART",
        "id": "bart",
        "path": "zoo:bart/bart_large/model",
        "agent": "bart",
        "external_website": "https://arxiv.org/abs/1910.13461",
        "task": "wikipedia_plus_toronto_books",
        "description": (
            "BART: Denoising Sequence-to-Sequence Pre-training for Natural "
            "Language Generation, Translation, and Comprehension."
        ),
        "example": (
            "parlai eval_model -mf zoo:bart/bart_large/model -t convai2 -bs 64"
        ),
        "result": (
            "Finished evaluating tasks ['convai2'] using datatype valid\n"
            "accuracy   bleu-4    exs      f1  gpu_mem    loss    ppl  token_acc   tpb\n"
            "0        .0004641   7801  .02084    .4878   5.041  154.6      .2042  1652"
        ),
        "example2": (
            "parlai train_model -m bart -mf /tmp/model_file -t convai2 -bs 24 --fp16 true -eps 1 -lr 1e-5 --optimizer adam"
        ),
        "result2": (
            "valid:\n"
            "accuracy  bleu-4  exs    f1  gpu_mem  loss    lr   ppl  token_acc  total_train_updates   tpb\n"
            ".0001282  .01229 7801 .2035    .6361 2.386 1e-05 10.87      .4741                 5478 321.3"
        ),
    },
    {
        "title": "Unlikelihood ConvAI2 context and label repetition model",
        "id": "dialogue_unlikelihood",
        "path": "zoo:dialogue_unlikelihood/rep_convai2_ctxt_and_label/model",
        "agent": "projects.dialogue_unlikelihood.agents:RepetitionUnlikelihoodAgent",
        "task": "convai2",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dialogue_unlikelihood",
        "description": (
            "Dialogue model finetuned on ConvAI2 with context and label repetition unlikelihood"
        ),
        "example": (
            "python parlai/scripts/interactive.py -mf zoo:dialogue_unlikelihood/rep_convai2_ctxt_and_label/model -m projects.dialogue_unlikelihood.agents:RepetitionUnlikelihoodAgent"
        ),
        "result": (
            "Enter Your Message: Hi.\n"
            "[RepetitionUnlikelihood]: hi , how are you doing today ?"
        ),
    },
    {
        "title": "Unlikelihood ConvAI2 context repetition model",
        "id": "dialogue_unlikelihood",
        "path": "zoo:dialogue_unlikelihood/rep_convai2_ctxt/model",
        "agent": "projects.dialogue_unlikelihood.agents:RepetitionUnlikelihoodAgent",
        "task": "convai2",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dialogue_unlikelihood",
        "description": (
            "Dialogue model finetuned on ConvAI2 with context repetition unlikelihood"
        ),
        "example": (
            "python parlai/scripts/interactive.py -mf zoo:dialogue_unlikelihood/rep_convai2_ctxt/model -m projects.dialogue_unlikelihood.agents:RepetitionUnlikelihoodAgent"
        ),
        "result": (
            "Enter Your Message: Hi.\n"
            "[RepetitionUnlikelihood]: hi , how are you doing today ?"
        ),
    },
    {
        "title": "Unlikelihood ConvAI2 label repetition model",
        "id": "dialogue_unlikelihood",
        "path": "zoo:dialogue_unlikelihood/rep_convai2_label/model",
        "agent": "projects.dialogue_unlikelihood.agents:RepetitionUnlikelihoodAgent",
        "task": "convai2",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dialogue_unlikelihood",
        "description": (
            "Dialogue model finetuned on ConvAI2 with label repetition unlikelihood"
        ),
        "example": (
            "python parlai/scripts/interactive.py -mf zoo:dialogue_unlikelihood/rep_convai2_label/model -m projects.dialogue_unlikelihood.agents:RepetitionUnlikelihoodAgent"
        ),
        "result": (
            "Enter Your Message: Hi.\n"
            "[RepetitionUnlikelihood]: hi , how are you doing today ?"
        ),
    },
    {
        "title": "Unlikelihood ELI5 context and label repetition model",
        "id": "dialogue_unlikelihood",
        "path": "zoo:dialogue_unlikelihood/rep_eli5_ctxt_and_label/model",
        "agent": "projects.dialogue_unlikelihood.agents:RepetitionUnlikelihoodAgent",
        "task": "eli5",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dialogue_unlikelihood",
        "description": (
            "Dialogue model finetuned on ELI5 with context and label repetition unlikelihood"
        ),
        "example": (
            "python parlai/scripts/interactive.py -mf zoo:dialogue_unlikelihood/rep_eli5_ctxt_and_label/model -m projects.dialogue_unlikelihood.agents:RepetitionUnlikelihoodAgent"
        ),
        "result": ("Enter Your Message: Hi.\n" "[RepetitionUnlikelihood]: hi ."),
    },
    {
        "title": "Unlikelihood ELI5 context repetition model",
        "id": "dialogue_unlikelihood",
        "path": "zoo:dialogue_unlikelihood/rep_celi5_ctxt/model",
        "agent": "projects.dialogue_unlikelihood.agents:RepetitionUnlikelihoodAgent",
        "task": "eli5",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dialogue_unlikelihood",
        "description": (
            "Dialogue model finetuned on ELI5 with context repetition unlikelihood"
        ),
        "example": (
            "python parlai/scripts/interactive.py -mf zoo:dialogue_unlikelihood/rep_eli5_ctxt/model -m projects.dialogue_unlikelihood.agents:RepetitionUnlikelihoodAgent"
        ),
        "result": ("Enter Your Message: Hi.\n" "[RepetitionUnlikelihood]: hi ."),
    },
    {
        "title": "Unlikelihood ELI5 label repetition model",
        "id": "dialogue_unlikelihood",
        "path": "zoo:dialogue_unlikelihood/rep_eli5_label/model",
        "agent": "projects.dialogue_unlikelihood.agents:RepetitionUnlikelihoodAgent",
        "task": "eli5",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dialogue_unlikelihood",
        "description": (
            "Dialogue model finetuned on ELI5 with label repetition unlikelihood"
        ),
        "example": (
            "python parlai/scripts/interactive.py -mf zoo:dialogue_unlikelihood/rep_eli5_label/model -m projects.dialogue_unlikelihood.agents:RepetitionUnlikelihoodAgent"
        ),
        "result": ("Enter Your Message: Hi.\n" "[RepetitionUnlikelihood]: hi ."),
    },
    {
        "title": "Unlikelihood Wizard of Wikipedia context and label repetition model",
        "id": "dialogue_unlikelihood",
        "path": "zoo:dialogue_unlikelihood/rep_wiki_ctxt_and_label/model",
        "agent": "projects.dialogue_unlikelihood.agents:RepetitionUnlikelihoodAgent",
        "task": "wizard_of_wikipedia:GeneratorTeacher",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dialogue_unlikelihood",
        "description": (
            "Dialogue model finetuned on Wizard of Wikipedia with context and label repetition unlikelihood"
        ),
        "example": (
            "python parlai/scripts/interactive.py -mf zoo:dialogue_unlikelihood/rep_wiki_ctxt_and_label/model -m projects.dialogue_unlikelihood.agents:RepetitionUnlikelihoodAgent"
        ),
        "result": ("Enter Your Message: Hi.\n" "[RepetitionUnlikelihood]: hi ."),
    },
    {
        "title": "Unlikelihood Wizard of Wikipedia context repetition model",
        "id": "dialogue_unlikelihood",
        "path": "zoo:dialogue_unlikelihood/rep_wiki_ctxt/model",
        "agent": "projects.dialogue_unlikelihood.agents:RepetitionUnlikelihoodAgent",
        "task": "wizard_of_wikipedia:GeneratorTeacher",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dialogue_unlikelihood",
        "description": (
            "Dialogue model finetuned on Wizard of Wikipedia with context repetition unlikelihood"
        ),
        "example": (
            "python parlai/scripts/interactive.py -mf zoo:dialogue_unlikelihood/rep_wiki_ctxt/model -m projects.dialogue_unlikelihood.agents:RepetitionUnlikelihoodAgent"
        ),
        "result": ("Enter Your Message: Hi.\n" "[RepetitionUnlikelihood]: hi ."),
    },
    {
        "title": "Unlikelihood Wizard of Wikipedia label repetition model",
        "id": "dialogue_unlikelihood",
        "path": "zoo:dialogue_unlikelihood/rep_wiki_label/model",
        "agent": "projects.dialogue_unlikelihood.agents:RepetitionUnlikelihoodAgent",
        "task": "wizard_of_wikipedia:GeneratorTeacher",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dialogue_unlikelihood",
        "description": (
            "Dialogue model finetuned on Wizard of Wikipedia with label repetition unlikelihood"
        ),
        "example": (
            "python parlai/scripts/interactive.py -mf zoo:dialogue_unlikelihood/rep_wiki_label/model -m projects.dialogue_unlikelihood.agents:RepetitionUnlikelihoodAgent"
        ),
        "result": ("Enter Your Message: Hi.\n" "[RepetitionUnlikelihood]: hi ."),
    },
    {
        "title": "Unlikelihood vocab alpha 1e0 model",
        "id": "dialogue_unlikelihood",
        "path": "zoo:dialogue_unlikelihood/vocab_alpha1e0/model",
        "agent": "projects.dialogue_unlikelihood.agents:RepetitionUnlikelihoodAgent",
        "task": "convai2",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dialogue_unlikelihood",
        "description": (
            "Dialogue model finetuned on convai2 with vocab unlikelihood, alpha value 1e0"
        ),
        "example": (
            "python parlai/scripts/interactive.py -mf zoo:dialogue_unlikelihood/vocab_alpha1e0/model -m projects.dialogue_unlikelihood.agents:TransformerSequenceVocabUnlikelihoodAgent"
        ),
        "result": (
            "Enter Your Message: Hi.\n"
            "[TransformerSequenceVocabUnlikelihood]: hi there ! how are you ?"
        ),
    },
    {
        "title": "Unlikelihood vocab alpha 1e1 model",
        "id": "dialogue_unlikelihood",
        "path": "zoo:dialogue_unlikelihood/vocab_alpha1e1/model",
        "agent": "projects.dialogue_unlikelihood.agents:RepetitionUnlikelihoodAgent",
        "task": "convai2",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dialogue_unlikelihood",
        "description": (
            "Dialogue model finetuned on convai2 with vocab unlikelihood, alpha value 1e1"
        ),
        "example": (
            "python parlai/scripts/interactive.py -mf zoo:dialogue_unlikelihood/vocab_alpha1e1/model -m projects.dialogue_unlikelihood.agents:TransformerSequenceVocabUnlikelihoodAgent"
        ),
        "result": (
            "Enter Your Message: Hi.\n"
            "[TransformerSequenceVocabUnlikelihood]: hi how are you today"
        ),
    },
    {
        "title": "Unlikelihood vocab alpha 1e2 model",
        "id": "dialogue_unlikelihood",
        "path": "zoo:dialogue_unlikelihood/vocab_alpha1e2/model",
        "agent": "projects.dialogue_unlikelihood.agents:RepetitionUnlikelihoodAgent",
        "task": "convai2",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dialogue_unlikelihood",
        "description": (
            "Dialogue model finetuned on convai2 with vocab unlikelihood, alpha value 1e2"
        ),
        "example": (
            "python parlai/scripts/interactive.py -mf zoo:dialogue_unlikelihood/vocab_alpha1e2/model -m projects.dialogue_unlikelihood.agents:TransformerSequenceVocabUnlikelihoodAgent"
        ),
        "result": (
            "Enter Your Message: Hi.\n"
            "[TransformerSequenceVocabUnlikelihood]: hello , how are you ?"
        ),
    },
    {
        "title": "Unlikelihood vocab alpha 1e3 model",
        "id": "dialogue_unlikelihood",
        "path": "zoo:dialogue_unlikelihood/vocab_alpha1e3/model",
        "agent": "projects.dialogue_unlikelihood.agents:RepetitionUnlikelihoodAgent",
        "task": "convai2",
        "project": "https://github.com/facebookresearch/ParlAI/tree/master/projects/dialogue_unlikelihood",
        "description": (
            "Dialogue model finetuned on convai2 with vocab unlikelihood, alpha value 1e3"
        ),
        "example": (
            "python parlai/scripts/interactive.py -mf zoo:dialogue_unlikelihood/vocab_alpha1e3/model -m projects.dialogue_unlikelihood.agents:TransformerSequenceVocabUnlikelihoodAgent"
        ),
        "result": (
            "Enter Your Message: Hi.\n"
            "[TransformerSequenceVocabUnlikelihood]: hi there !"
        ),
    },
    {
        "title": "Style-controlled generation: C75-D+ generator",
        "id": "style_gen",
        "path": "zoo:style_gen/c75_labeled_dialogue_generator/model",
        "agent": "projects.style_gen.style_gen:StyleGenAgent",
        "task": "style_gen:LabeledBlendedSkillTalk",
        "project": 'https://github.com/facebookresearch/ParlAI/tree/master/projects/style_gen',
        "description": "Generator trained on dialogue datasets, with 75% of train examples appended with Image-Chat personality labels",
        "example": "parlai eval_model --datatype test --model projects.style_gen.style_gen:StyleGenAgent --model-file zoo:style_gen/c75_labeled_dialogue_generator/model --skip-generation True --task style_gen:LabeledBlendedSkillTalk --use-style-frac 1.00",
        "result": """16:56:52 | Finished evaluating tasks ['style_gen:LabeledBlendedSkillTalk'] using datatype test
    ctpb  ctps  exps  exs  gpu_mem  loss  ltpb  ltps   ppl  token_acc   tpb  tps
     120  1855 15.46 5482    .1635 2.248 19.94 308.2 9.468      .4872 139.9 2163""",
    },
    {
        "title": "Style-controlled generation: previous and current utterance classifier",
        "id": "style_gen",
        "path": "zoo:style_gen/prev_curr_classifier/model",
        "agent": "projects.style_gen.classifier:ClassifierAgent",
        "task": "style_gen:LabeledBlendedSkillTalk",
        "project": 'https://github.com/facebookresearch/ParlAI/tree/master/projects/style_gen',
        "description": "Classifier trained on Image-Chat turns 2 and 3 to classify the personality of an example given that utterance and the previous utterance.",
        "example": "parlai eval_model --task style_gen:PrevCurrUttStyle --wrapper-task style_gen:LabeledBlendedSkillTalk --model-file zoo:style_gen/prev_curr_classifier/model --model projects.style_gen.classifier:ClassifierAgent --classes-from-file image_chat_personalities_file",
        "result": """18:42:33 | Finished evaluating tasks ['style_gen:PrevCurrUttStyle'] using datatype valid
    accuracy  bleu-4  ctpb  ctps  exps  exs    f1  gpu_mem  loss    lr  ltpb  ltps   tpb   tps
       .9973  .01745 38.08 604.1 15.86 5651 .9973    .1622 2.129 5e-10 5.633 89.36 43.71 693.4""",
    },
    {
        "title": "Faster-R-CNN Detectron Features",
        "id": "detectron",
        "path": "zoo:detectron/detectron_model.pth",
        "agent": "n/a",
        "task": "n/a",
        "project": 'https://github.com/facebookresearch/vilbert-multi-task',
        "description": "Detectron Model for extracting 100x2048d object detection features. Model is from linked project website",
        "example": "parlai display_data -t flickr30k --image-mode faster_r_cnn_152_32x8d -n 1 -v",
        "result": """
        [id]: flickr30k
        [image_id]: 1000092795
        [ image ]: tensor([[ 0.0000,  7.1329,  0.0000,  ...,  0.0000,  0.0000,  5.3357],
                [ 0.0000,  0.0000,  0.0000,  ...,  0.0000,  0.0000,  0.0000],
                [ 0.0000,  0.0000,  0.4687,  ...,  0.0000,  0.0000,  0.0000],
                ...,
                [ 0.0000,  3.0936,  3.4888,  ...,  0.0000,  0.0000,  0.0000],
                [ 3.8596,  0.0000,  0.0000,  ...,  0.0000,  4.3454,  0.0000],
                [ 0.0000,  7.9822,  1.0979,  ...,  3.5514,  0.0000, 15.3559]])
        [labels]: Two young guys with shaggy hair look at their hands while hanging out in the yard.|Two young, White males are outside near many bushes.|Two men in green shirts are standing in a yard.|A man in a blue shirt standing in a garden.|Two friends enjoy time spent together.
        """,
    },
    {
        "title": "Multi-Modal BlenderBot (MMB DegenPos)",
        "id": "multimodal_blenderbot",
        "path": 'n/a',
        "agent": "projects.multimodal_blenderbot.agents:BiasAgent",
        "task": "blended_skill_talk",
        "project": 'https://github.com/facebookresearch/ParlAI/tree/master/projects/multimodal_blenderbot',
        "description": "Model trained to talk about both images and general chitchat, trained with a degendering teacher and with 75% of Image-Chat styles replaced by a generic polarity string",
        "example": "python parlai/scripts/safe_interactive.py -t blended_skill_talk -mf ${FINETUNED_MODEL_PATH} --model projects.multimodal_blenderbot.agents:BiasAgent --delimiter $'\n' --beam-block-ngram 3 --beam-context-block-ngram 3 --beam-min-length 20 --beam-size 10 --inference beam --model-parallel False",
        "result": "(results will vary)",
    },
    {
        "title": "Transformer Classifier Multi-turn Dialogue Safety Model",
        "id": "bot_adversarial_dialogue",
        "path": "zoo:bot_adversarial_dialogue/multi_turn_v0/model",
        "agent": "transformer/classifier",
        "task": "bot_adversarial_dialogue",
        "project": "",
        "description": (
            "Classifier trained on the filtered multi-turn bot adversarial dialogues in addition to both dialogue_safety single-turn standard and adversarial safety tasks and Wikipedia Toxic Comments."
        ),
        "example": (
            "parlai eval_model -t bot_adversarial_dialogue:bad_num_turns=4 -dt test -mf zoo:bot_adversarial_dialogue/multi_turn_v0/model -bs 128"
        ),
        "result": (
            "{'exs': 2598, 'accuracy': 0.8414, 'f1': 0.8414, 'loss': 0.5153, 'bleu-4': 8.414e-10, 'class___notok___recall': 0.8093, 'class___notok___prec': 0.7671, 'class___notok___f1': 0.7876, 'class___ok___recall': 0.8597, 'class___ok___prec': 0.8876, 'class___ok___f1': 0.8735, 'weighted_f1': 0.8423}"
        ),
    },
    {
        "title": "Transformer Classifier Multi-turn Dialogue Safety Model",
        "id": "bot_adversarial_dialogue",
        "path": "zoo:bot_adversarial_dialogue/multi_turn/model",
        "agent": "transformer/classifier",
        "task": "bot_adversarial_dialogue",
        "project": "",
        "description": (
            "Classifier trained on the truncated multi-turn bot adversarial dialogues in addition to both dialogue_safety single-turn standard and adversarial safety tasks and Wikipedia Toxic Comments."
        ),
        "example": (
            "parlai eval_model -t bot_adversarial_dialogue:bad_num_turns=4 -dt test -mf zoo:bot_adversarial_dialogue/multi_turn/model -bs 128"
        ),
        "result": (
            "{'exs': 2598, 'accuracy': 0.8507, 'f1': 0.8507, 'loss': 0.3878, 'bleu-4': 8.507e-10, 'class___notok___recall': 0.8633, 'class___notok___prec': 0.7588, 'class___notok___f1': 0.8077, 'class___ok___recall': 0.8434, 'class___ok___prec': 0.9154, 'class___ok___f1': 0.8779, 'weighted_f1': 0.8524}"
        ),
    },
    {
        "title": "Transformer Classifier Sensitive Topics Detection",
        "id": "sensitive_topics_classifier",
        "path": "zoo:sensitive_topics_classifier/model",
        "agent": "transformer/classifier",
        "task": "sensitive_topics_evaluation",
        "project": "",
        "description": (
            "Pretrained Transformer-based classifier for classification of sensitive topics."
        ),
        "example": (
            "parlai eval_model -mf zoo:sensitive_topics_classifier/model -t sensitive_topics_evaluation -dt valid -bs 16"
        ),
        "result": (
            "{'exs': 1811, 'accuracy': 0.7332965212589729, 'f1': 0.7332965212589729, 'bleu-4': 7.332965212589829e-10, 'loss': 0.8489562001990587, 'class_none_prec': 0.0, 'class_none_recall': 0.0, 'class_none_f1': 0.0, 'class_politics_prec': 0.8762376237623762, 'class_politics_recall': 0.885, 'class_politics_f1': 0.8805970149253731, 'class_religion_prec': 0.8829568788501027, 'class_religion_recall': 0.8669354838709677, 'class_religion_f1': 0.8748728382502543, 'class_medical_prec': 0.8237704918032787, 'class_medical_recall': 0.7077464788732394, 'class_medical_f1': 0.7613636363636364, 'class_nsfw_prec': 0.7769784172661871, 'class_nsfw_recall': 0.32142857142857145, 'class_nsfw_f1': 0.45473684210526316, 'class_drugs_prec': 0.8901515151515151, 'class_drugs_recall': 0.7966101694915254, 'class_drugs_f1': 0.8407871198568873, 'weighted_f1': 0.774835331736443}"
        ),
    },
    {
        "title": "MDGender Bert Ranker Classifier",
        "id": "md_gender",
        "path": "zoo:md_gender/model",
        "agent": "projects.md_gender.bert_ranker_classifier.agents:BertRankerClassifierAgent",
        "task": "md_gender",
        "project": "",
        "description": (
            "Ranker/classifier agent trained to predict gender bias along several axes using the MD Gender task."
        ),
        "example": (
            "parlai interactive -t md_gender -m projects.md_gender.bert_ranker_classifier.agents:BertRankerClassifierAgent -mf zoo:md_gender/model -ecands inline -cands inline --interactive_mode False --data-parallel False"
        ),
        "result": (
            "Enter Your Message: Hi my name is Emily!\n"
            "[MDGender Classifier]: SELF: female"
        ),
    },
]



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.
"""
Save a pickle of images and associated contexts for the model image chat task.
"""

import os
import pickle

from parlai.agents.repeat_label.repeat_label import RepeatLabelAgent
from parlai.core.opt import Opt
from parlai.core.worlds import create_task
from parlai.crowdsourcing.tasks.model_chat import run
from parlai.crowdsourcing.tasks.model_chat.utils import (
    get_context_generator,
    get_image_src,
)
from parlai.scripts.display_data import setup_args


def setup_image_context_args():
    task_parser = setup_args()
    default_image_context_path = os.path.join(
        os.path.dirname(run.__file__), 'task_config', 'image_contexts'
    )
    task_parser.add_argument(
        '--image-context-path',
        type=str,
        default=default_image_context_path,
        help='Save path for image context file',
    )
    return task_parser


def save_image_contexts(task_opt: Opt):
    """
    Save a JSON of images and associated contexts for the model image chat task.

    Note that each image will have BST-style context information saved with it, such as
    persona strings and a pair of lines of dialogue from another dataset.
    TODO: perhaps have the image chat task make use of this context information
    """

    print('Creating teacher to loop over images.')
    agent = RepeatLabelAgent(task_opt)
    world = create_task(task_opt, agent)
    num_examples = task_opt['num_examples']

    print('Creating context generator.')
    context_generator = get_context_generator()

    print(f'Looping over {num_examples:d} images and pulling a context for each one.')
    image_contexts = []
    unique_image_srcs = set()
    while len(image_contexts) < num_examples:

        # Get the next teacher act
        world.parley()
        teacher_act = world.get_acts()[0]

        image_src = get_image_src(image=teacher_act['image'])
        if image_src in unique_image_srcs:
            # Skip over non-unique images, such as from the later turns of an episode
            print('\tSkipping non-unique image.')
        else:
            unique_image_srcs.add(image_src)
            image_context = {
                'image_act': teacher_act,
                'context_info': context_generator.get_context(),
            }
            image_contexts.append(image_context)
            if len(image_contexts) % 5 == 0:
                print(f'Collected {len(image_contexts):d} images.')

    print(f'{len(image_contexts):d} image contexts created.')

    # Save
    with open(task_opt['image_context_path'], 'wb') as f:
        pickle.dump(image_contexts, f)


if __name__ == '__main__':
    task_opt_ = setup_image_context_args().parse_args()
    save_image_contexts(task_opt_)



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.


import parlai.core.build_data as build_data
import os
from parlai.core.build_data import DownloadableFile

ROOT_URL = 'https://github.com/google-research-datasets/Taskmaster/raw/master/TM-2-2020'

RESOURCES = [
    # raw data files
    DownloadableFile(
        f'{ROOT_URL}/data/flights.json',
        'flights.json',
        '86b37b5ae25f530fd18ced78800d30c3b54f7b34bb208ecb51842718f04e760b',
        zipped=False,
    ),
    DownloadableFile(
        f'{ROOT_URL}/data/food-ordering.json',
        'food-ordering.json',
        '0a042e566a816a5d0abebe6f7e8cfd6abaa89729ffc42f433d327df7342b12f8',
        zipped=False,
    ),
    DownloadableFile(
        f'{ROOT_URL}/data/hotels.json',
        'hotels.json',
        '975b0242f1e37ea1ab94ccedd7e0d6ee5831599d5df1f16143e71110d6c6006a',
        zipped=False,
    ),
    DownloadableFile(
        f'{ROOT_URL}/data/movies.json',
        'movies.json',
        '6f67c9a1f04abc111186e5bcfbe3050be01d0737fd6422901402715bc1f3dd0d',
        zipped=False,
    ),
    DownloadableFile(
        f'{ROOT_URL}/data/music.json',
        'music.json',
        'e5db60d6576fa010bef87a70a8b371d293d48cde8524c1d3ed7c3022f079d95d',
        zipped=False,
    ),
    DownloadableFile(
        f'{ROOT_URL}/data/restaurant-search.json',
        'restaurant-search.json',
        'fb9735f89e7ebc7c877f976da4c30391af6a6277991b597c0755564657ff8f47',
        zipped=False,
    ),
    DownloadableFile(
        f'{ROOT_URL}/data/sports.json',
        'sports.json',
        '8191531bfa5a8426b1508c396ab9886a19c7c620b443c436ec10d8d4708d0eac',
        zipped=False,
    ),
    # ontology data files
    DownloadableFile(
        f'{ROOT_URL}/ontology/flights.json',
        'flights.onto.json',
        '1ebc5c982339d24b2dcf50677883fed65b7fcb95f01edbbd3be6357090893c33',
        zipped=False,
    ),
    DownloadableFile(
        f'{ROOT_URL}/ontology/food-ordering.json',
        'food-ordering.onto.json',
        '79c1189c16f0ab937bad558c70a0b9b99358f9ed91ea65ce4af37c4b7d999063',
        zipped=False,
    ),
    DownloadableFile(
        f'{ROOT_URL}/ontology/hotels.json',
        'hotels.onto.json',
        '22ae51ba546ee7ca03143097782817c4cdd0de74ac84893eaf40b8254aa866d3',
        zipped=False,
    ),
    DownloadableFile(
        f'{ROOT_URL}/ontology/movies.json',
        'movies.onto.json',
        '8403283526bb314e871850b98bb86a7987ef0af6fbbe4fb5a089ee9498584476',
        zipped=False,
    ),
    DownloadableFile(
        f'{ROOT_URL}/ontology/music.json',
        'music.onto.json',
        '4bcd6dcf1cdc6bdb717e5fdc08b3472dc3d1f4da8a0f8aee917494d79a7fe338',
        zipped=False,
    ),
    DownloadableFile(
        f'{ROOT_URL}/ontology/restaurant-search.json',
        'restaurant-search.onto.json',
        'c9ead7985695b3feba1fb955e8407d806e4095f5459485adc5448ae89989e609',
        zipped=False,
    ),
    DownloadableFile(
        f'{ROOT_URL}/ontology/sports.json',
        'sports.onto.json',
        '52f9bbb86ebd9e2b3916185ad4e3e9b8b77d2164d96bd3b98ad67cbaa653757d',
        zipped=False,
    ),
]


def build(opt):
    # get path to data directory
    dpath = os.path.join(opt['datapath'], 'taskmaster-2')
    # define version if any
    version = "1.1"

    # check if data had been previously built
    if not build_data.built(dpath, version_string=version):
        print('[building data: ' + dpath + ']')

        # make a clean directory if needed
        if build_data.built(dpath):
            # an older version exists, so remove these outdated files.
            build_data.remove_dir(dpath)
        build_data.make_dir(dpath)

        # Download the data.
        for downloadable_file in RESOURCES:
            downloadable_file.download_file(dpath)

        # mark the data as built
        build_data.mark_done(dpath, version_string=version)



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

from parlai.core.worlds import World


# ----- Baseline overworld that simply defers to the default world ----- #
class SimpleMessengerOverworld(World):
    """
    Passthrough world to spawn task worlds of only one type.

    Demos of more advanced overworld functionality exist in the overworld demo
    """

    def __init__(self, opt, agent):
        self.agent = agent
        self.opt = opt
        self.episodeDone = False

    def episode_done(self):
        return self.episodeDone

    @staticmethod
    def generate_world(opt, agents):
        return SimpleMessengerOverworld(opt, agents[0])

    def parley(self):
        self.episodeDone = True
        return 'default'


class OnboardWorld(World):
    def __init__(self, opt, agent):
        self.agent = agent
        self.episodeDone = False

    @staticmethod
    def generate_world(opt, agents):
        return OnboardWorld(opt, agents[0])

    def parley(self):
        self.episodeDone = True

    def episode_done(self):
        return self.episodeDone

    def shutdown(self):
        pass



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

"""
Custom flake8 rules for ParlAI.

Includes:
- Checks for python3 shebang
- Check for copyright message
- Docformatter issues (TODO)
"""

import docformatter
import difflib

PYTHON_SHEBANG = '#!/usr/bin/env python3'
ALLOWLIST_PHRASES = ['Moscow Institute of Physics and Technology.']
ALLOWLIST_FNS = ["mlb_vqa"]
COPYRIGHT = [
    "Copyright (c) Facebook, Inc. and its affiliates.",
    "This source code is licensed under the MIT license found in the",
    "LICENSE file in the root directory of this source tree.",
]


class ParlAIChecker:
    """
    Custom flake8 checker for some special ParlAI requirements.
    """

    name = 'flake8-parlai'
    version = '0.1'

    def __init__(self, tree=None, filename=None, lines=None):
        self.filename = filename
        self.lines = lines

    def run(self):
        if self.lines is None:
            with open(self.filename) as f:
                self.lines = f.readlines()

        if self.lines and PYTHON_SHEBANG not in self.lines[0]:
            yield (
                1,
                0,
                'PAI100 Missing python3 shebang. (`#!/usr/bin/env python3`)',
                '',
            )

        # check doc formatting
        source = "".join(self.lines)
        formatted_source = docformatter.format_code(
            source,
            pre_summary_newline=True,
            description_wrap_length=88,
            summary_wrap_length=88,
            make_summary_multi_line=True,
            force_wrap=False,
        )
        if source != formatted_source:
            diff = difflib.unified_diff(
                source.split('\n'),  # have to strip newlines
                formatted_source.split('\n'),
                f'before/{self.filename}',
                f'after/{self.filename}',
                n=0,
                lineterm='',
            )
            for line in diff:
                if line.startswith('@@'):
                    fields = line.split()
                    # find out the beginning line of the docstring reformat. Example:
                    # --- /path/to/original  timestamp
                    # +++ /path/to/new       timestamp
                    # @@ -1,3 +1,9 @@
                    # that -1 says the first line changed, and 3 lines were removed
                    # with a new offset belonging at the first line, and 9
                    # inserted lines.
                    line_no, *_ = fields[1].split(',')
                    line_no = -int(line_no)
                    yield (
                        line_no,
                        1,
                        'PAI101 autoformat.sh would reformat the docstring',
                        '',
                    )

        # the rest is checking copyright, but there are some exceptions
        # copyright must appear in the first 16 lines of the file.
        source = "".join(self.lines[:16])
        if any(wl in source for wl in ALLOWLIST_PHRASES):
            return

        for i, msg in enumerate(COPYRIGHT, 1):
            if any(wl in self.filename for wl in ALLOWLIST_FNS) and i < 3:
                continue
            if source and msg not in source:
                yield (i, 0, f'PAI20{i} Missing copyright `{msg}`', '')



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.
"""
Messenger Manager Module.

Contains implementation of the MessengerManager, which helps run ParlAI via FB
Messenger.
"""
import logging
import os

from parlai.core.agents import create_agent
import parlai.chat_service.utils.logging as log_utils
import parlai.chat_service.utils.server as server_utils
from parlai.utils.io import PathManager
from parlai.chat_service.services.messenger.agents import MessengerAgent
from parlai.chat_service.core.socket import ChatServiceMessageSocket
from parlai.chat_service.services.messenger.message_sender import MessageSender
from parlai.chat_service.core.chat_service_manager import ChatServiceManager

parent_dir = os.path.dirname(os.path.abspath(__file__))


class MessengerManager(ChatServiceManager):
    """
    Manages interactions between agents on messenger as well as direct interactions
    between agents and the messenger overworld.
    """

    def __init__(self, opt):
        """
        Create an MessengerManager using the given setup options.
        """
        # Manager attributes
        super().__init__(opt)

        self._init_logs()
        # Read in Config
        self._parse_config(opt)
        self._complete_setup()

    def parse_additional_args(self, opt):
        self.service_reference_id = self.config['additional_args']['page_id']
        self.should_load_model = self.config['additional_args'].get('load_model', True)
        if self.service_reference_id == 1:
            raise RuntimeError(
                'Please configure your own page in order to run this task. '
                'See the docs (https://parl.ai/docs/tutorial_messenger.html) '
                'for more information.'
            )

    def restructure_message(self, message):
        message.update(message.get('message', {}))
        return message

    def _complete_setup(self):
        """
        Complete necessary setup items.
        """
        self.setup_server()
        self.init_new_state()
        self.setup_socket()
        self.start_new_run()
        self._load_model()

    def _load_model(self):
        """
        Load model if necessary.
        """
        if 'models' in self.opt and self.should_load_model:
            model_params = {}
            model_info = {}
            for model in self.opt['models']:
                model_opt = self.opt['models'][model]
                overrides = model_opt.get('overrides', {})
                if type(overrides) is list:
                    model_opt['overrides'] = overrides[0]
                model_params[model] = create_agent(model_opt).share()
                model_info[model] = {'overrides': overrides}
            self.runner_opt['model_info'] = model_info
            self.runner_opt['shared_bot_params'] = model_params

    def _init_logs(self):
        """
        Initialize logging settings from the opt.
        """
        log_utils.set_is_debug(self.opt['is_debug'])
        log_utils.set_log_level(self.opt['log_level'])

    def mark_removed(self, agent_id, pageid):
        """
        Mark the agent as removed from the pool.

        Can be overriden to change other metadata linked to agent removal.

        :param agent_id:
            int agent psid
        :param pageid:
            int page id
        """
        pass

    def _handle_bot_read(self, agent_id):
        self.sender.send_read(agent_id)
        self.sender.typing_on(agent_id)

    def _confirm_message_delivery(self, event):
        # By default we don't actually do anything when messages are marked as
        # being delivered, but we expose the ability for others to
        self._log_debug(
            'Messages {} marked as received.'.format(event['delivery']['mids'])
        )

    def _handle_message_read(self, event):
        # If the message was sent by another user (as in during a conversation)
        # then we need to propogate the read back to that user.
        self._log_debug('Messages {} marked as read.'.format(event['read']))
        super()._handle_message_read(event)

    def _handle_webhook_event(self, event):
        if 'message' in event:
            if ('image_url' in event and event['image_url'] is not None) or (
                'attachment_url' in event and event['attachment_url'] is not None
            ):
                event['message']['image'] = True
            self._on_new_message(event)
        elif 'delivery' in event:
            self.confirm_message_delivery(event)
        elif 'read' in event:
            self.handle_message_read(event)

    def after_agent_removed(self, agent_id):
        """
        Perform any changes to metadata on agent removal.

        override if extra bookkeeping must be done when removing agent
        """
        pass

    def _create_agent(self, task_id, agent_id):
        """
        Initialize an agent and return it.

        Called each time an agent is placed into a new task.

        :param task_id:
            string task identifier
        :param agent_id:
            int agent id
        """
        return MessengerAgent(
            self.opt, self, task_id, agent_id, self.service_reference_id
        )

    def _log_missing_agent(self, agent_id, assignment_id):
        """
        Log the occurence of a missing agent.
        """
        log_utils.print_and_log(
            logging.WARN,
            'Expected to have an agent for {}_{}, yet none was found'.format(
                agent_id, assignment_id
            ),
        )

    # Manager Lifecycle Functions #
    def setup_server(self):
        """
        Prepare the Messenger server for handling messages.
        """
        if self.bypass_server_setup:
            return

        log_utils.print_and_log(
            logging.INFO,
            '\nYou are going to allow people on Facebook to be agents in '
            'ParlAI.\nDuring this process, Internet connection is required, '
            'and you should turn off your computer\'s auto-sleep '
            'feature.\n',
            should_print=True,
        )
        input('Please press Enter to continue... ')
        log_utils.print_and_log(logging.NOTSET, '', True)

        if self.opt['local'] is True:
            log_utils.print_and_log(
                logging.INFO,
                'In order to run the server locally, you will need '
                'to have a public HTTPS endpoint (SSL signed) running on '
                'the server you are currently excecuting ParlAI on. Enter '
                'that public URL hostname when prompted and ensure that the '
                'port being used by ParlAI (usually 3000) has external '
                'traffic routed to it.',
                should_print=True,
            )
            input('Please press Enter to continue... ')

        log_utils.print_and_log(
            logging.INFO, 'Setting up Messenger webhook...', should_print=True
        )

        # Setup the server with a task name related to the current task
        task_name = '{}-{}'.format('ParlAI-Messenger', self.opt['task'])
        self.server_task_name = ''.join(
            e for e in task_name.lower() if e.isalnum() or e == '-'
        )
        self.server_url = server_utils.setup_server(
            self.server_task_name, local=self.opt['local']
        )
        log_utils.print_and_log(
            logging.INFO,
            'Webhook address: {}/webhook'.format(self.server_url),
            should_print=True,
        )

    # override if permission needed externally
    def get_app_token(self):
        """
        Find and return an app access token.
        """
        if not self.opt.get('force_page_token'):
            if not os.path.exists(os.path.expanduser('~/.parlai/')):
                PathManager.mkdirs(os.path.expanduser('~/.parlai/'))
            access_token_file_path = '~/.parlai/messenger_token'
            expanded_file_path = os.path.expanduser(access_token_file_path)
            if os.path.exists(expanded_file_path):
                with open(expanded_file_path, 'r') as access_token_file:
                    return access_token_file.read()

        token = input(
            'Enter your page\'s access token from the developer page at'
            'https://developers.facebook.com/apps/<YOUR APP ID>'
            '/messenger/settings/ to continue setup:'
        )
        access_token_file_path = '~/.parlai/messenger_token'
        expanded_file_path = os.path.expanduser(access_token_file_path)
        with open(expanded_file_path, 'w+') as access_token_file:
            access_token_file.write(token)
        return token

    def setup_socket(self):
        """
        Set up socket to start communicating to workers.
        """
        if self.bypass_server_setup:
            return

        log_utils.print_and_log(
            logging.INFO, 'Local: Setting up WebSocket...', should_print=True
        )

        self.app_token = self.get_app_token()
        self.sender = MessageSender(self.app_token)

        # Set up receive
        socket_use_url = self.server_url
        if self.opt['local']:  # skip some hops for local stuff
            socket_use_url = 'https://localhost'
        self.socket = ChatServiceMessageSocket(
            socket_use_url, self.port, self._handle_webhook_event
        )
        log_utils.print_and_log(logging.INFO, 'done with websocket', should_print=True)

    # Agent Interaction Functions #

    def observe_message(self, receiver_id, text, quick_replies=None, persona_id=None):
        """
        Send a message through the message manager.

        :param receiver_id:
            int identifier for agent to send message to
        :param text:
            string text to send
        :param quick_replies:
            list of quick replies
        :param persona_id:
            identifier of persona
        """
        return self.sender.send_fb_message(
            receiver_id, text, True, quick_replies=quick_replies, persona_id=persona_id
        )

    def observe_payload(self, receiver_id, data, quick_replies=None, persona_id=None):
        """
        Send a payload through the message manager.

        :param receiver_id:
            int identifier for agent to send message to
        :param data:
            object data to send
        :param quick_replies:
            list of quick replies
        :param persona_id:
            identifier of persona
        """
        return self.sender.send_fb_payload(
            receiver_id, data, quick_replies=quick_replies, persona_id=persona_id
        )

    def upload_attachment(self, payload):
        """
        Upload an attachment and return an attachment ID.

        :param payload:
            dict with the following format:
                {'type': <TYPE>, 'url': <URL>} or
                {'type': <TYPE>, 'filename': <FILENAME>, 'format': <FILEFORMAT>}.
                For example,
                {'type': 'image', 'filename': 'test.png', 'format': 'png'}
        """
        return self.sender.upload_fb_attachment(payload)



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

import copy
import json
import os
from datetime import datetime
from typing import Any, Dict, Optional, Tuple

import numpy as np
import pandas as pd

from parlai.crowdsourcing.utils.analysis import AbstractTurnAnnotationResultsCompiler


class TurnAnnotationsStaticResultsCompiler(AbstractTurnAnnotationResultsCompiler):
    """
    Class to compile results from static turn annotations.

    Change PROBLEM_BUCKETS in task_config/annotation_buckets.json to be the buckets that
    you are asking crowdsource workers to annotate with.
    """

    NUM_SUBTASKS = 7
    LIVE_ONBOARDING_IS_LAST_SUBTASK = True
    LIVE_ONBOARDING_THRESHOLD = 0.5
    INFLIGHT_ONBOARDING_DATA = None
    NUM_ANNOTATIONS = 5

    FILENAME_STUB = 'results'
    CALCULATE_STATS_INTERANNOTATOR_AGREEMENT = True

    @classmethod
    def setup_args(cls):
        parser = super().setup_args()
        parser.add_argument(
            '--onboarding-in-flight-data-file',
            type=str,
            help='Path to JSONL file containing onboarding in-flight conversations',
        )
        parser.add_argument(
            '--gold-annotations-file',
            type=str,
            default=None,
            help='Path to a JSON file mapping utterance IDs to the gold annotations',
        )
        return parser

    def __init__(self, opt: Dict[str, Any]):
        super().__init__(opt)
        self.onboarding_in_flight_data_file = opt.get('onboarding_in_flight_data_file')
        self.gold_annotations_file = opt.get('gold_annotations_file')

    def get_data_paths_mephisto(self, task_run_id_folder):
        """
        Get all the individual folders with data from the <task_run_id> path we are
        given as input.

        In Mephisto the structure is:
        /<project_id>/<task_run_id>/<assignment_id>/<agent_id>/

        Side note: assignment_id == HIT ID
        """
        # TODO: replace direct folder access with a call to
        #  mephisto.tools.data_browser.DataBrowser
        read_folders = []
        for assignment_id in os.listdir(task_run_id_folder):
            if assignment_id in ['onboarding', 'reservations', 'build', '.', '..']:
                continue
            assignment_folder = os.path.join(task_run_id_folder, assignment_id)
            if os.path.isdir(assignment_folder):
                if len(os.listdir(assignment_folder)) > 2:
                    print(
                        f'Had more than one HIT in folder: {assignment_folder}, had {len(os.listdir(assignment_folder))} folders.'
                    )
                for agent_id in os.listdir(assignment_folder):
                    if os.path.isdir(os.path.join(assignment_folder, agent_id)):
                        full_path = os.path.join(
                            task_run_id_folder,
                            assignment_id,
                            agent_id,
                            'agent_data.json',
                        )
                        read_folders.append(full_path)
        return read_folders

    def get_results_path_base(self) -> str:
        now = datetime.now()
        return os.path.join(
            self.output_folder, f'{self.FILENAME_STUB}_{now.strftime("%Y%m%d_%H%M%S")}'
        )

    def compile_results(self) -> pd.DataFrame:
        # Loads data from files and gets rid of incomplete or malformed convos
        conversations = self.compile_initial_results(self.results_folders)
        master_dataframe = self.process_data_into_dataframe(conversations)
        self.calculate_basic_interannotator_agreement(master_dataframe)
        if self.gold_annotations_file is not None:
            with open(self.gold_annotations_file, 'r') as gold_f:
                gold_annotations = json.loads(gold_f.read())
                self.calculate_agreement_with_gold_annotations(
                    gold_annotations, master_dataframe
                )
        if self.CALCULATE_STATS_INTERANNOTATOR_AGREEMENT:
            self.calculate_stats_interannotator_agreement(master_dataframe)

        return master_dataframe

    def _validate_hit(self, hit_data) -> Tuple[bool, Optional[str]]:
        """
        Validate an entire HIT.

        :return: tuple (is_valid, reason)
        """
        if 'outputs' not in hit_data or hit_data['outputs'] is None:
            return False, 'Malformed HIT'

        subtasks = hit_data['outputs']['final_data']
        if len(subtasks) != self.NUM_SUBTASKS:
            return False, f'Incomplete HIT with subtask length {len(subtasks)}.'

        return True, None

    def _validate_subtask(self, subtask_data) -> Tuple[bool, Optional[str]]:
        """
        Validate a conversation subtask within the HIT.

        :return: tuple (is_valid, reason)
        """
        # Check that the conversation consists of pairs of comments between
        # agents 0 and 1, with 0 speaking first
        try:
            assert all(
                [
                    utterance_data['agent_idx'] == turn_idx % 2
                    for turn_idx, utterance_data in enumerate(subtask_data)
                ]
            )
            messages_0 = [utt for utt in subtask_data if utt['agent_idx'] == 0]
            messages_1 = [utt for utt in subtask_data if utt['agent_idx'] == 1]
            assert len(messages_0) + len(messages_1) == len(subtask_data)
        except Exception:
            return False, f'Data not in form expected. Length is: {len(subtask_data)}'

        for utterance_data in subtask_data:
            if (
                utterance_data['agent_idx'] == 1
                and self.problem_buckets[0] not in utterance_data
            ):
                return (
                    False,
                    f'Bot utterance was malformed and had no problem annotation fields (Failed to find key: {self.problem_buckets[0]}).',
                )

        return True, None

    def _get_inflight_onboarding_success_from_subtask(self, subtask):
        if self.INFLIGHT_ONBOARDING_DATA is None:
            self.INFLIGHT_ONBOARDING_DATA = self.setup_inflight_onboarding_data()
        onboarding_utterance = subtask['data'][-1]
        num_answers = 0
        num_correct = 0
        num_incorrect = 0
        for d in self.INFLIGHT_ONBOARDING_DATA:
            if d['dialog'][-1][-1]['text'] == onboarding_utterance['text']:
                num_answers = len(d['answers'])
                for pb in self.problem_buckets:
                    if pb in d['answers'] and onboarding_utterance[pb]:
                        num_correct += 1
                    if onboarding_utterance[pb] and pb not in d['answers']:
                        num_incorrect += 1
        return num_correct / num_answers

    def compile_initial_results(self, results_folders) -> list:
        """
        Do initial loading and processing of crowdsource data Loads data from all the
        worker ID files and gets rid of incomplete or malformed convos.

        Also adds fields such as worker_id, assignment_id, etc for convenience
        :return: list of JSON objects which represent a conversation with
        annotations d["data"] of each has an array of utterance level data
        """
        print('Starting compile_initial_results...')
        all_data_paths = []
        for f in results_folders:
            # Each one is a HIT completed by a given worker (so if
            # units-per-assignment > 1), then will include the same conversations
            # multiple times annotated by different workers
            data_paths = self.get_data_paths_mephisto(f)
            all_data_paths.extend(data_paths)
        print(f'Got {len(all_data_paths)} folders to read.')

        conversations = []
        task_completion_times = []
        for dp in all_data_paths:
            # Read in file
            with open(os.path.join(dp), 'rb') as f:
                data = json.load(f)

            worker_id = dp.split('/')[-2]
            hit_id = dp.split('/')[-3]
            _ = dp.split('/')[-4]  # Task run

            (is_valid_hit, reason) = self._validate_hit(data)
            if not is_valid_hit:
                print(
                    f'Skipping invalid HIT {hit_id}, worker_id: {worker_id} for reason: {reason}.'
                )
                continue

            # HIT-level metric of HIT completion time has to be done here for now
            task_completion_time_seconds = (
                data['times']['task_end'] - data['times']['task_start']
            )
            print(task_completion_time_seconds)

            subtasks = data['outputs']['final_data']
            if self.LIVE_ONBOARDING_IS_LAST_SUBTASK:
                qc_success_pct = self._get_inflight_onboarding_success_from_subtask(
                    subtasks[-1]
                )
            else:
                qc_success_pct = 0.0
            for subtask_idx, d in enumerate(subtasks):
                if (
                    subtask_idx == (len(subtasks) - 1)
                    and self.LIVE_ONBOARDING_IS_LAST_SUBTASK
                ):
                    # Last subtask is inflight onboarding; don't include it
                    continue
                subtask_data = copy.deepcopy(d)
                # Structure of each subtask is {'subtask_index': XX, 'data': [...]}
                (is_valid_subtask, reason) = self._validate_subtask(d['data'])
                if not is_valid_subtask:
                    print(
                        f'Skipping invalid subtask within HIT: {hit_id}, worker_id: {worker_id} for reason: {reason}.'
                    )
                    continue

                subtask_data['worker_id'] = worker_id
                subtask_data['hit_id'] = hit_id
                subtask_data['folder'] = dp
                subtask_data['subtask_idx'] = subtask_idx
                experimental_design = 'self_chat'
                subtask_data['model_nickname'] = experimental_design + '/' + 'TODO'
                subtask_data['qc_success_pct'] = qc_success_pct
                conversations.append(subtask_data)

            task_completion_times.append(task_completion_time_seconds)

        if len(task_completion_times) > 0:
            print(
                f'Average task completion time (seconds) was: '
                f'{np.average(task_completion_times):0.1f}'
            )

        if len(conversations) == 0:
            raise ValueError('No conversations found!')

        return conversations

    def process_data_into_dataframe(self, conversations) -> pd.DataFrame:
        """
        Return one big dataframe of all conversations where a row is an utterance and
        its problem annotations.
        """
        print('Starting process_data_into_dataframe...')
        rows = []
        for _, convo in enumerate(conversations):
            for turn_idx, utt in enumerate(convo['data']):
                row = {
                    'annotation_id': f'{convo["hit_id"]}_{convo["subtask_idx"]}_{turn_idx}_{convo["worker_id"]}',
                    'conversation_id': f'{convo["hit_id"]}_{convo["subtask_idx"]}',
                    'utterance_id': f'{convo["hit_id"]}_{convo["subtask_idx"]}_{turn_idx}',
                    'turn_idx': turn_idx,
                    'agent_idx': utt['agent_idx'],
                    'folder': convo['folder'],
                    'worker_id': convo['worker_id'],
                    'hit_id': convo['hit_id'],
                    'model_nickname': convo['model_nickname'],
                    'qc_success_pct': convo['qc_success_pct'],
                    'text': utt['text'],
                }
                row = self._add_additional_columns(row=row, utt=utt)
                for k in self.problem_buckets:
                    row[k] = utt[k] if utt['agent_idx'] == 1 else ''
                rows.append(row)
        df = pd.DataFrame(rows)
        print(f'Returning master dataframe with {len(df)} annotations.')
        return df

    def _add_additional_columns(self, row: Dict[str, Any], utt: dict) -> Dict[str, Any]:
        """
        Add additional columns to the results dataframe.

        If you wish to add additional columns to the results dataframe, use the input
        utterance dict `utt` to define new keys in `row`, which will form one row in the
        final results dataframe.
        """
        _ = utt
        # The utt dict isn't used here, but may be used in subclasses.
        return row

    def calculate_basic_interannotator_agreement(self, df: pd.DataFrame) -> None:
        print('Starting calculate_interannotator_agreement...')
        # Drops the human utterances which don't have problem buckets
        bot_only_df = df.replace('', np.nan)
        # Get rid of the None's, which occur if there were no checkboxes (so if
        # last utterance only option selected)
        bot_only_df = bot_only_df.fillna(value=np.nan)
        bot_only_df = bot_only_df.dropna()

        bot_only_df = self._problem_bucket_specific_filtering(bot_only_df)

        # Group at the utterance level (summing across workers)
        bot_only_df = bot_only_df.replace(True, 1)
        bot_only_df = bot_only_df.replace(False, 0)
        all_bot_annotations_count = len(bot_only_df)

        # Remove utterances that don't have self.NUM_ANNOTATIONS annotations
        counted_df = bot_only_df.groupby(['utterance_id']).count()
        counted_df = counted_df[counted_df == self.NUM_ANNOTATIONS].dropna()
        bot_only_df = bot_only_df[bot_only_df['utterance_id'].isin(counted_df.index)]
        print(
            f'Removed {all_bot_annotations_count - len(bot_only_df)} that did not have annotations by {self.NUM_ANNOTATIONS} workers. {len(bot_only_df)} annotations remaining.'
        )

        if self.LIVE_ONBOARDING_IS_LAST_SUBTASK:
            # Remove those that didn't get enough right on live onboarding
            bot_only_df = bot_only_df[
                bot_only_df['qc_success_pct'] >= self.LIVE_ONBOARDING_THRESHOLD
            ]

        summed_df = bot_only_df.groupby(['utterance_id']).sum()
        print(f'summed_df has length {len(summed_df)}; bot_only_df: {len(bot_only_df)}')

        utterance_ids = bot_only_df['utterance_id'].unique()
        print(f'Number of unique utterance_ids: {len(utterance_ids)}.')

        if 'any_problem' in summed_df:
            # We've computed a column marking if any problem exists, so include this
            extended_problem_buckets = self.problem_buckets + ['any_problem']
        else:
            extended_problem_buckets = self.problem_buckets
        for k in extended_problem_buckets:
            one_annotator = len(summed_df[summed_df[k] == 1])
            two_annotators = len(summed_df[summed_df[k] == 2])
            three_annotators = len(summed_df[summed_df[k] >= 3])
            total_problem_annotations = (
                one_annotator + two_annotators + three_annotators
            )
            total_utterances = len(summed_df[k])
            if total_problem_annotations > 0:
                print(
                    f'Bucket: {k}, total unique problem utterances: {total_problem_annotations} ({total_problem_annotations/total_utterances:.1%} of all), one annotator: {one_annotator} ({one_annotator/total_problem_annotations:.1%}), two_annotators: {two_annotators} ({two_annotators/total_problem_annotations:.1%}), three+ annotators: {three_annotators} ({three_annotators/total_problem_annotations:.1%})'
                )

    def _problem_bucket_specific_filtering(
        self, bot_only_df: pd.DataFrame
    ) -> pd.DataFrame:
        """
        Filter the bot responses given the specific problem buckets being used.
        """

        non_none_problem_buckets = [
            bucket for bucket in self.problem_buckets if bucket != 'none_all_good'
        ]
        assert len(set(non_none_problem_buckets)) + 1 == len(self.problem_buckets)
        # Make sure problem buckets are all unique

        utterance_count_total = len(bot_only_df)

        # Select which columns are consistent
        is_consistent = (
            bot_only_df[non_none_problem_buckets[0]] + bot_only_df['none_all_good']
        ) < 2
        for bucket in non_none_problem_buckets[1:]:
            is_consistent = is_consistent & (
                (bot_only_df[bucket] + bot_only_df['none_all_good']) < 2
            )
        bot_only_df['is_consistent'] = is_consistent
        bot_only_df = bot_only_df[bot_only_df['is_consistent']]

        # If any of the problem buckets have marked True
        any_problem = bot_only_df[non_none_problem_buckets[0]]
        for bucket in non_none_problem_buckets[1:]:
            any_problem = any_problem | bot_only_df[bucket]
        bot_only_df['any_problem'] = any_problem

        print(
            f'Dropped {utterance_count_total - len(bot_only_df)} inconsistently annotated utterances (none_all_good and a problem bucket). Now have {len(bot_only_df)} utterances.'
        )

        return bot_only_df

    def calculate_agreement_with_gold_annotations(
        self, gold_annotations, df: pd.DataFrame
    ) -> None:
        """
        Assume gold_annotations are a dictionary of the form {utterance_id : {bucket_i:
        true/false}} where utterance_id is taken from the compile_initial_results (i.e.
        mephistohitid_subtaskindex_utteranceidx)
        """
        print('Starting calculate_agreement_with_gold_annotations...')
        # Drops the human utterances which don't have problem buckets
        bot_only_df = df.replace('', np.nan)
        # Get rid of the None's, which occur if there were no checkboxes (so if
        # last utterance only option selected)
        bot_only_df = bot_only_df.fillna(value=np.nan)
        bot_only_df = bot_only_df.dropna()

        # Include only utterances that have gold_annotations
        bot_only_df = bot_only_df[
            bot_only_df['utterance_id'].isin(gold_annotations.keys())
        ]

        print(
            f'Got {len(gold_annotations.keys())} utterances with gold annotations. Found {len(bot_only_df)} utterances matching gold annotations from DataFrame.'
        )

        agreement_map = {pb: [] for pb in self.problem_buckets}
        agreement_map_problem_only = {pb: [] for pb in self.problem_buckets}
        problem_counts = {pb: 0 for pb in self.problem_buckets}
        for utterance_id, gold in gold_annotations.items():
            utterance_df = bot_only_df[bot_only_df['utterance_id'] == utterance_id]
            count_workers = len(utterance_df)

            for pb in self.problem_buckets:
                gold_annotation = gold[pb]
                match_count = utterance_df[utterance_df[pb] == gold[pb]].count()[pb]
                a = float(match_count / count_workers)
                agreement_map[pb].append(a)
                if gold_annotation:
                    agreement_map_problem_only[pb].append(a)
                    problem_counts[pb] += 1
        print(
            f'------------------------\nAverage agreement with {len(gold_annotations)} total gold utterances annotated was:'
        )
        for pb in self.problem_buckets:
            print(
                f'{pb}: {np.average(agreement_map[pb]):.1%} ({problem_counts[pb]} gold problem samples)'
            )
        print('------------------------')
        print(
            f'------------------------\nAverage agreement problem samples only with {len(gold_annotations)} total gold utterances annotated was:'
        )
        for pb in self.problem_buckets:
            print(
                f'{pb}: {np.average(agreement_map_problem_only[pb]):.1%} ({problem_counts[pb]} gold problem samples)'
            )
        print('------------------------')

    def calculate_stats_interannotator_agreement(self, df: pd.DataFrame):
        print('Starting calculate_stats_interannotator_agreement...')
        # Get rid of the human utterances (non-annotated)
        bot_only_df = df.replace('', np.nan)
        # Get rid of the None's, which occur if there were no checkboxes (so if
        # last utterance only option selected)
        bot_only_df = bot_only_df.fillna(value=np.nan)
        bot_only_df = bot_only_df.dropna()
        print(f'Calculating agreement on {len(bot_only_df)} annotations.')

        for pb in self.problem_buckets:
            # Expects a df of rater_id, item_id and "data" column
            kappa_df = df[['annotation_id', 'worker_id', 'utterance_id', pb]]
            kappa_df = kappa_df.rename(
                columns={'worker_id': 'rater_id', 'utterance_id': 'item_id', pb: 'data'}
            )
            try:
                fleiss_kappa = self.compute_fleiss_kappa(
                    kappa_df, [True, False], self.NUM_ANNOTATIONS
                )
            except Exception as exc:
                print(f'Exception calculating Fleiss Kappa: {exc}. Skipping.')
                continue
            print(f'Fleiss\' kappa for {pb} is: {fleiss_kappa:0.3f}.')

    def compute_fleiss_kappa(
        self, df: pd.DataFrame, categories: list, number_of_raters: int
    ) -> float:
        """
        Expects a df of index, rater_id, item_id and "data" column with each row a label
        of one of the categories.
        """

        # categories are "True" and "False"
        # As per wikipedia definition: https://en.wikipedia.org/wiki/Fleiss%27_kappa
        items_list = df.drop_duplicates(subset=['item_id'])['item_id'].to_list()
        N = len(items_list)
        p_j = np.zeros(len(categories))
        P_bar_sum_term = 0.0
        for item in items_list:
            df_item_annotations = df[df['item_id'] == item]
            if len(df_item_annotations) != number_of_raters:
                continue
            for j, c in enumerate(categories):
                try:
                    n_ij = df_item_annotations['data'].value_counts()[c]
                except Exception:
                    n_ij = 0.0
                p_j[j] += n_ij
                P_bar_sum_term += n_ij ** 2

        p_j = [tmp / (N * number_of_raters) for tmp in p_j]
        P_e_bar = sum([tmp ** 2 for tmp in p_j])

        P_bar = (P_bar_sum_term - N * number_of_raters) / (
            N * number_of_raters * (number_of_raters - 1)
        )

        kappa = (P_bar - P_e_bar) / (1 - P_e_bar)
        return kappa

    def setup_inflight_onboarding_data(self):
        print('setup_inflight_onboarding_data')
        raw_qc_convos = []
        with open(self.onboarding_in_flight_data_file, "r") as f:
            line = f.readline()
            while line:
                qc_convo = json.loads(line)
                raw_qc_convos.append(qc_convo)
                line = f.readline()
        return raw_qc_convos


if __name__ == '__main__':
    parser_ = TurnAnnotationsStaticResultsCompiler.setup_args()
    args = parser_.parse_args()
    TurnAnnotationsStaticResultsCompiler(vars(args)).compile_and_save_results()



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.
"""
Count and display statistics of the data.

## Examples

```shell
parlai data_stats --task convai2
```
"""
from parlai.core.params import ParlaiParser
from parlai.agents.fixed_response.fixed_response import FixedResponseAgent
from parlai.core.worlds import create_task
from parlai.utils.misc import TimeLogger, nice_report
from parlai.core.metrics import AverageMetric
from parlai.core.dict import DictionaryAgent
from parlai.core.script import ParlaiScript, register_script

import parlai.utils.logging as logging


def setup_args(parser=None):
    if parser is None:
        parser = ParlaiParser(True, False, 'Compute data statistics')
    # Get command line arguments
    parser.add_argument('-n', '-ne', '--num-examples', type=int, default=-1)
    parser.add_argument('-ltim', '--log-every-n-secs', type=float, default=10)
    parser.add_argument(
        '--agent',
        type=int,
        default=0,
        help='Use teacher (agent 0) or model (agent 1)',
        choices=[0, 1],
    )
    parser.add_argument(
        '--new_line_new_utt',
        type='bool',
        default=False,
        help='New lines treat substrings as separate utterances.',
    )
    parser.add_argument(
        '--ignore_tokens',
        type=str,
        default='',
        help='ignore tokens containings these substrings (comma-separated)',
    )
    parser.set_defaults(datatype='train:ordered')
    DictionaryAgent.add_cmdline_args(parser, partial_opt=None)
    return parser


def _report(world, counts):
    report = world.report()
    for k, v in counts.items():
        if not isinstance(v, dict):
            report[k] = v
    return report


def verify(opt):
    if opt['datatype'] == 'train':
        logging.warn('changing datatype from train to train:ordered')
        opt['datatype'] = 'train:ordered'

    # create repeat label agent and assign it to the specified task
    opt['fixed_response'] = None
    agent = FixedResponseAgent(opt)
    world = create_task(opt, agent)
    opt.log()

    log_every_n_secs = opt.get('log_every_n_secs', -1)
    if log_every_n_secs <= 0:
        log_every_n_secs = float('inf')
    log_time = TimeLogger()

    dictionary = DictionaryAgent(opt)
    ignore_tokens = opt.get('ignore_tokens').split(',')

    counts = {}
    for t in {'input', 'labels', 'both'}:
        counts[f'{t}/tokens'] = 0
        counts[f'{t}/utterances'] = 0
        counts[f'{t}/avg_utterance_length'] = None
        counts[f'{t}/unique_tokens'] = 0
        counts[f'{t}/unique_utterances'] = 0
        # for counting the stats..
        counts[f'{t}/token_dict'] = {}
        counts[f'{t}/utterance_dict'] = {}

    def tokenize(txt):
        return dictionary.tokenize(txt)

    def keep_token(t):
        for s in ignore_tokens:
            if s != '' and s in t:
                return False
        return True

    # max number of examples to evaluate
    max_cnt = opt['num_examples'] if opt['num_examples'] > 0 else float('inf')
    cnt = 0

    # Show some example dialogs.
    while not world.epoch_done() and world.total_exs < max_cnt:
        world.parley()
        act = world.get_acts()[opt.get('agent')]
        for itype in {'input', 'labels'}:
            if itype == 'input':
                if opt.get('new_line_new_utt'):
                    txts = act.get('text').split('\n')
                else:
                    txts = [act.get('text')]
            else:
                txts = act.get('labels', act.get('eval_labels', ['']))

            for txt in txts:
                tokens = tokenize(txt)
                retxt = [t for t in tokens if keep_token(t)]
                counts[f'{itype}/tokens'] += len(retxt)
                counts['both/tokens'] += len(retxt)
                counts[f'{itype}/utterances'] += 1
                counts['both/utterances'] += 1
                counts[f'{itype}/avg_utterance_length'] += AverageMetric(len(retxt), 1)
                counts[f'both/avg_utterance_length'] += AverageMetric(len(retxt), 1)
                for t in retxt:
                    if t not in counts[f'{itype}/token_dict']:
                        counts[f'{itype}/unique_tokens'] += 1
                        counts[f'{itype}/token_dict'][t] = True
                    if t not in counts['both/token_dict']:
                        counts['both/unique_tokens'] += 1
                        counts['both/token_dict'][t] = True
                retxt = ' '.join(retxt)
                if retxt not in counts[f'{itype}/utterance_dict']:
                    counts[f'{itype}/unique_utterances'] += 1
                    counts[f'{itype}/utterance_dict'][retxt] = True
                if retxt not in counts['both/utterance_dict']:
                    counts['both/unique_utterances'] += 1
                    counts['both/utterance_dict'][retxt] = True

        if log_time.time() > log_every_n_secs:
            report = _report(world, counts)
            cnt = report.pop('exs')
            text, log = log_time.log(cnt, world.num_examples(), report)
            logging.info(text)

    try:
        # print dataset size if available
        logging.info(
            f'loaded {world.num_episodes()} episodes with a total '
            f'of {world.num_examples()} examples'
        )
    except AttributeError:
        pass

    retval = _report(world, counts)
    retval.pop('exs')
    return retval


def obtain_stats(opt):
    report = verify(opt)
    print(nice_report(report))
    return report


@register_script('data_stats', hidden=True)
class DataStats(ParlaiScript):
    @classmethod
    def setup_args(cls):
        return setup_args()

    def run(self):
        return obtain_stats(self.opt)


if __name__ == '__main__':
    DataStats.main()



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.
"""
Pretrained model allowing to get the same performances as in
https://arxiv.org/abs/1905.01969.
"""
from parlai.core.build_data import built, download_models, get_model_dir
import os
import os.path


def download(datapath):
    model_name = 'pretrained_transformers'
    mdir = os.path.join(get_model_dir(datapath), model_name)
    version = 'v3.0'
    if not built(mdir, version):
        opt = {'datapath': datapath}
        fnames = ['pretrained_transformers.tgz']
        download_models(opt, fnames, model_name, version=version, use_model_type=False)
